@@ -37,6 +37,8 @@ sub _check_params {
         'oblique_title',
         'font',
         'font_size',
+        'scale_width',
+        'scale_height',
         'callnum_split',
         'justify',
         'format_string',
@@ -223,32 +225,61 @@ sub _BIB {
 
 sub _BAR {
     my $self = shift;
-    my $barcode_llx = $self->{'llx'} + $self->{'left_text_margin'};     # this places the bottom left of the barcode the left text margin distance to right of the left edge of the label ($llx)
-    my $barcode_lly = $self->{'lly'} + $self->{'top_text_margin'};      # this places the bottom left of the barcode the top text margin distance above the bottom of the label ($lly)
-    my $barcode_width = 0.8 * $self->{'width'};                         # this scales the barcode width to 80% of the label width
-    my $barcode_y_scale_factor = 0.01 * $self->{'height'};              # this scales the barcode height to 10% of the label height
+    my $barcode_llx =
+        $self->{'llx'} +
+        $self->{'left_text_margin'}
+        ; # this places the bottom left of the barcode the left text margin distance to right of the left edge of the label ($llx)
+    my $barcode_lly =
+        $self->{'lly'} +
+        $self->{'top_text_margin'}
+        ; # this places the bottom left of the barcode the top text margin distance above the bottom of the label ($lly)
+    my $barcode_width = $self->{'scale_width'} * $self->{'width'}
+        ;    # You can choose the width of barcode, default value is 0.8 : 80% of the label width
+    my $barcode_y_scale_factor = $self->{'scale_height'} * $self->{'height'}
+        ;    # You can choose the height of barcode, default value is 0.01 : 10% of the label height
     return 0, 0, 0, $barcode_llx, $barcode_lly, $barcode_width, $barcode_y_scale_factor;
 }
 
 sub _BIBBAR {
     my $self = shift;
-    my $barcode_llx = $self->{'llx'} + $self->{'left_text_margin'};     # this places the bottom left of the barcode the left text margin distance to right of the left edge of the label ($self->{'llx'})
-    my $barcode_lly = $self->{'lly'} + $self->{'top_text_margin'};      # this places the bottom left of the barcode the top text margin distance above the bottom of the label ($lly)
-    my $barcode_width = 0.8 * $self->{'width'};                         # this scales the barcode width to 80% of the label width
-    my $barcode_y_scale_factor = 0.01 * $self->{'height'};              # this scales the barcode height to 10% of the label height
-    my $line_spacer = ($self->{'font_size'} * 1);       # number of pixels between text rows (This is actually leading: baseline to baseline minus font size. Recommended starting point is 20% of font size.).
-    my $text_lly = ($self->{'lly'} + ($self->{'height'} - $self->{'top_text_margin'}));
+    my $barcode_llx =
+        $self->{'llx'} +
+        $self->{'left_text_margin'}
+        ; # this places the bottom left of the barcode the left text margin distance to right of the left edge of the label ($self->{'llx'})
+    my $barcode_lly =
+        $self->{'lly'} +
+        $self->{'top_text_margin'}
+        ; # this places the bottom left of the barcode the top text margin distance above the bottom of the label ($lly)
+    my $barcode_width = $self->{'scale_width'} * $self->{'width'}
+        ;    # You can choose the width of barcode, default value is 0.8 : 80% of the label width
+    my $barcode_y_scale_factor = $self->{'scale_height'} * $self->{'height'}
+        ;    # You can choose the height of barcode, default value is 0.01 : 10% of the label height
+    my $line_spacer = ( $self->{'font_size'} * 1 )
+        ; # number of pixels between text rows (This is actually leading: baseline to baseline minus font size. Recommended starting point is 20% of font size.).
+    my $text_lly = ( $self->{'lly'} + ( $self->{'height'} - $self->{'top_text_margin'} ) );
     return $self->{'llx'}, $text_lly, $line_spacer, $barcode_llx, $barcode_lly, $barcode_width, $barcode_y_scale_factor;
 }
 
 sub _BARBIB {
     my $self = shift;
-    my $barcode_llx = $self->{'llx'} + $self->{'left_text_margin'};                             # this places the bottom left of the barcode the left text margin distance to right of the left edge of the label ($self->{'llx'})
-    my $barcode_lly = ($self->{'lly'} + $self->{'height'}) - $self->{'top_text_margin'};        # this places the bottom left of the barcode the top text margin distance below the top of the label ($self->{'lly'})
-    my $barcode_width = 0.8 * $self->{'width'};                                                 # this scales the barcode width to 80% of the label width
-    my $barcode_y_scale_factor = 0.01 * $self->{'height'};                                      # this scales the barcode height to 10% of the label height
-    my $line_spacer = ($self->{'font_size'} * 1);                               # number of pixels between text rows (This is actually leading: baseline to baseline minus font size. Recommended starting point is 20% of font size.).
-    my $text_lly = (($self->{'lly'} + $self->{'height'}) - $self->{'top_text_margin'} - (($self->{'lly'} + $self->{'height'}) - $barcode_lly));
+    my $barcode_llx =
+        $self->{'llx'} +
+        $self->{'left_text_margin'}
+        ; # this places the bottom left of the barcode the left text margin distance to right of the left edge of the label ($self->{'llx'})
+    my $barcode_lly =
+        ( $self->{'lly'} + $self->{'height'} ) -
+        $self->{'top_text_margin'}
+        ; # this places the bottom left of the barcode the top text margin distance below the top of the label ($self->{'lly'})
+    my $barcode_width = $self->{'scale_width'} * $self->{'width'}
+        ;    # You can choose the width of barcode, default value is 0.8 : 80% of the label width
+    my $barcode_y_scale_factor = $self->{'scale_height'} * $self->{'height'}
+        ;    # You can choose the height of barcode, default value is 0.01 : 10% of the label height
+    my $line_spacer = ( $self->{'font_size'} * 1 )
+        ; # number of pixels between text rows (This is actually leading: baseline to baseline minus font size. Recommended starting point is 20% of font size.).
+    my $text_lly =
+        ( ( $self->{'lly'} + $self->{'height'} ) -
+            $self->{'top_text_margin'} -
+            ( ( $self->{'lly'} + $self->{'height'} ) - $barcode_lly ) );
     return $self->{'llx'}, $text_lly, $line_spacer, $barcode_llx, $barcode_lly, $barcode_width, $barcode_y_scale_factor;
 }
 
@@ -256,26 +287,29 @@ sub new {
     my ($invocant, %params) = @_;
     my $type = ref($invocant) || $invocant;
     my $self = {
-        batch_id                => $params{'batch_id'},
-        item_number             => $params{'item_number'},
-        llx                     => $params{'llx'},
-        lly                     => $params{'lly'},
-        height                  => $params{'height'},
-        width                   => $params{'width'},
-        top_text_margin         => $params{'top_text_margin'},
-        left_text_margin        => $params{'left_text_margin'},
-        barcode_type            => $params{'barcode_type'},
-        printing_type           => $params{'printing_type'},
-        guidebox                => $params{'guidebox'},
-        oblique_title           => $params{'oblique_title'},
-        font                    => $params{'font'},
-        font_size               => $params{'font_size'},
-        callnum_split           => $params{'callnum_split'},
-        justify                 => $params{'justify'},
-        format_string           => $params{'format_string'},
-        text_wrap_cols          => $params{'text_wrap_cols'},
-        barcode                 => $params{'barcode'},
+        batch_id         => $params{'batch_id'},
+        item_number      => $params{'item_number'},
+        llx              => $params{'llx'},
+        lly              => $params{'lly'},
+        height           => $params{'height'},
+        width            => $params{'width'},
+        top_text_margin  => $params{'top_text_margin'},
+        left_text_margin => $params{'left_text_margin'},
+        barcode_type     => $params{'barcode_type'},
+        printing_type    => $params{'printing_type'},
+        guidebox         => $params{'guidebox'},
+        oblique_title    => $params{'oblique_title'},
+        font             => $params{'font'},
+        font_size        => $params{'font_size'},
+        scale_width      => $params{'scale_width'},
+        scale_height     => $params{'scale_height'},
+        callnum_split    => $params{'callnum_split'},
+        justify          => $params{'justify'},
+        format_string    => $params{'format_string'},
+        text_wrap_cols   => $params{'text_wrap_cols'},
+        barcode          => $params{'barcode'},
     };
+
     if ($self->{'guidebox'}) {
         $self->{'guidebox'} = _guide_box($self->{'llx'}, $self->{'lly'}, $self->{'width'}, $self->{'height'});
     }
