@@ -42,15 +42,10 @@ BEGIN {
     require Exporter;
     @ISA = qw(Exporter);
     @EXPORT_OK = qw(
-      GetAllIssues
-
       GetBorrowersToExpunge
 
       IssueSlip
 
-      get_cardnumber_length
-      checkcardnumber
-
       DeleteUnverifiedOpacRegistrations
       DeleteExpiredOpacRegistrations
     );
@@ -234,118 +229,6 @@ sub patronflags {
     return ( \%flags );
 }
 
-=head2 GetAllIssues
-
-  $issues = &GetAllIssues($borrowernumber, $sortkey, $limit);
-
-Looks up what the patron with the given borrowernumber has borrowed,
-and sorts the results.
-
-C<$sortkey> is the name of a field on which to sort the results. This
-should be the name of a field in the C<issues>, C<biblio>,
-C<biblioitems>, or C<items> table in the Koha database.
-
-C<$limit> is the maximum number of results to return.
-
-C<&GetAllIssues> an arrayref, C<$issues>, of hashrefs, the keys of which
-are the fields from the C<issues>, C<biblio>, C<biblioitems>, and
-C<items> tables of the Koha database.
-
-=cut
-
-#'
-sub GetAllIssues {
-    my ( $borrowernumber, $order, $limit ) = @_;
-
-    return unless $borrowernumber;
-    $order = 'date_due desc' unless $order;
-
-    my $dbh = C4::Context->dbh;
-    my $query =
-'SELECT issues.*, items.*, biblio.*, biblioitems.*, issues.timestamp as issuestimestamp, issues.renewals_count AS renewals,items.renewals AS totalrenewals,items.timestamp AS itemstimestamp,borrowers.firstname,borrowers.surname
-  FROM issues
-  LEFT JOIN items on items.itemnumber=issues.itemnumber
-  LEFT JOIN borrowers on borrowers.borrowernumber=issues.issuer_id
-  LEFT JOIN biblio ON items.biblionumber=biblio.biblionumber
-  LEFT JOIN biblioitems ON items.biblioitemnumber=biblioitems.biblioitemnumber
-  WHERE issues.borrowernumber=?
-  UNION ALL
-  SELECT old_issues.*, items.*, biblio.*, biblioitems.*, old_issues.timestamp as issuestimestamp, old_issues.renewals_count AS renewals,items.renewals AS totalrenewals,items.timestamp AS itemstimestamp,borrowers.firstname,borrowers.surname
-  FROM old_issues
-  LEFT JOIN items on items.itemnumber=old_issues.itemnumber
-  LEFT JOIN borrowers on borrowers.borrowernumber=old_issues.issuer_id
-  LEFT JOIN biblio ON items.biblionumber=biblio.biblionumber
-  LEFT JOIN biblioitems ON items.biblioitemnumber=biblioitems.biblioitemnumber
-  WHERE old_issues.borrowernumber=? AND old_issues.itemnumber IS NOT NULL
-  order by ' . $order;
-    if ($limit) {
-        $query .= " limit $limit";
-    }
-
-    my $sth = $dbh->prepare($query);
-    $sth->execute( $borrowernumber, $borrowernumber );
-    return $sth->fetchall_arrayref( {} );
-}
-
-sub checkcardnumber {
-    my ( $cardnumber, $borrowernumber ) = @_;
-
-    # If cardnumber is null, we assume they're allowed.
-    return 0 unless defined $cardnumber;
-
-    my $dbh = C4::Context->dbh;
-    my $query = "SELECT * FROM borrowers WHERE cardnumber=?";
-    $query .= " AND borrowernumber <> ?" if ($borrowernumber);
-    my $sth = $dbh->prepare($query);
-    $sth->execute(
-        $cardnumber,
-        ( $borrowernumber ? $borrowernumber : () )
-    );
-
-    return 1 if $sth->fetchrow_hashref;
-
-    my ( $min_length, $max_length ) = get_cardnumber_length();
-    return 2
-        if length $cardnumber > $max_length
-        or length $cardnumber < $min_length;
-
-    return 0;
-}
-
-=head2 get_cardnumber_length
-
-    my ($min, $max) = C4::Members::get_cardnumber_length()
-
-Returns the minimum and maximum length for patron cardnumbers as
-determined by the CardnumberLength system preference, the
-BorrowerMandatoryField system preference, and the width of the
-database column.
-
-=cut
-
-sub get_cardnumber_length {
-    my $borrower = Koha::Database->new->schema->resultset('Borrower');
-    my $field_size = $borrower->result_source->column_info('cardnumber')->{size};
-    my ( $min, $max ) = ( 0, $field_size ); # borrowers.cardnumber is a nullable varchar(20)
-    $min = 1 if C4::Context->preference('BorrowerMandatoryField') =~ /cardnumber/;
-    if ( my $cardnumber_length = C4::Context->preference('CardnumberLength') ) {
-        # Is integer and length match
-        if ( $cardnumber_length =~ m|^\d+$| ) {
-            $min = $max = $cardnumber_length
-                if $cardnumber_length >= $min
-                    and $cardnumber_length <= $max;
-        }
-        # Else assuming it is a range
-        elsif ( $cardnumber_length =~ m|(\d*),(\d*)| ) {
-            $min = $1 if $1 and $min < $1;
-            $max = $2 if $2 and $max > $2;
-        }
-
-    }
-    $min = $max if $min > $max;
-    return ( $min, $max );
-}
-
 =head2 GetBorrowersToExpunge
 
   $borrowers = &GetBorrowersToExpunge(
@@ -399,6 +282,7 @@ sub GetBorrowersToExpunge {
     $query .= q| WHERE  category_type <> 'S'
         AND ( borrowers.flags IS NULL OR borrowers.flags = 0 )
         AND tmp.guarantor_id IS NULL
+        AND borrowers.protected = 0
     |;
     my @query_params;
     if ( $filterbranch && $filterbranch ne "" ) {
@@ -578,40 +462,24 @@ sub IssueSlip {
                 issues      => $all,
             };
         }
-        my $news = Koha::AdditionalContents->search_for_display(
+        my @news_ids = Koha::AdditionalContents->search_for_display(
             {
                 category   => 'news',
                 location   => 'slip',
                 lang       => $patron->lang,
                 library_id => $branch,
             }
-        );
-        my @news;
-        while ( my $n = $news->next ) {
-            my $all = $n->unblessed_all_relateds;
-
-            # FIXME We keep newdate and timestamp for backward compatibility (from GetNewsToDisplay)
-            # But we should remove them and adjust the existing templates in a db rev
-            # FIXME This must be formatted in the notice template
-            my $published_on_dt = output_pref({ dt => dt_from_string( $all->{published_on} ), dateonly => 1 });
-            $all->{newdate} = $published_on_dt;
-            $all->{timestamp} = $published_on_dt;
-
-            push @news, {
-                additional_contents => $all,
-            };
-        }
+        )->get_column('id');
         $letter_code = 'ISSUESLIP';
         %repeat      = (
             checkedout => \@checkouts,
             overdue    => \@overdues,
-            news       => \@news,
         );
         %loops = (
             issues => [ map { $_->{issues}{itemnumber} } @checkouts ],
             overdues   => [ map { $_->{issues}{itemnumber} } @overdues ],
-            opac_news => [ map { $_->{additional_contents}{idnew} } @news ],
-            additional_contents => [ map { $_->{additional_contents}{idnew} } @news ],
+            opac_news => \@news_ids,
+            additional_contents => \@news_ids,
         );
     }
 
