@@ -38,6 +38,7 @@ use Koha::SimpleMARC qw( read_field );
 use Koha::XSLT::Base;
 use Koha::CsvProfiles;
 use Koha::AuthorisedValues;
+use Koha::TemplateUtils qw( process_tt );
 use Carp qw( carp croak );
 
 use vars qw(@ISA @EXPORT);
@@ -564,13 +565,10 @@ sub marcrecord2csv {
 
         # TT tags exist
         if ( $content =~ m|\[\%.*\%\]| ) {
-            my $tt = Template->new();
-            my $template = $content;
             # Replace 00X and 0XX with X or XX
             $content =~ s|fields.00(\d)|fields.$1|g;
             $content =~ s|fields.0(\d{2})|fields.$1|g;
-            my $tt_output;
-            $tt->process( \$content, $field_list, \$tt_output );
+            my $tt_output = process_tt( $content, $field_list );
             push @csv_rows, $tt_output;
         } else {
             for my $tag ( @$tags ) {
@@ -578,9 +576,12 @@ sub marcrecord2csv {
                 # If it is a subfield
                 my @loop_values;
                 if (defined $tag->{subfieldtag} ) {
-                    my $av = Koha::AuthorisedValues->search_by_marc_field({ frameworkcode => $frameworkcode, tagfield => $tag->{fieldtag}, tagsubfield => $tag->{subfieldtag}, });
-                    $av = $av->count ? $av->unblessed : [];
-                    my $av_description_mapping = { map { ( $_->{authorised_value} => $_->{lib} ) } @$av };
+                    my $av_description_mapping = Koha::AuthorisedValues->get_descriptions_by_marc_field(
+                        {
+                            frameworkcode => $frameworkcode, tagfield => $tag->{fieldtag},
+                            tagsubfield   => $tag->{subfieldtag},
+                        }
+                    );
                     # For each field
                     foreach my $field (@fields) {
                         my @subfields = $field->subfield( $tag->{subfieldtag} );
@@ -591,21 +592,26 @@ sub marcrecord2csv {
 
                 # Or a field
                 } else {
-                    my $av = Koha::AuthorisedValues->search_by_marc_field({ frameworkcode => $frameworkcode, tagfield => $tag->{fieldtag}, });
-                    $av = $av->count ? $av->unblessed : [];
-                    my $authvalues = { map { ( $_->{authorised_value} => $_->{lib} ) } @$av };
 
                     foreach my $field ( @fields ) {
                         my $value;
 
                         # If it is a control field
                         if ($field->is_control_field) {
+                            my $authvalues = Koha::AuthorisedValues->get_descriptions_by_marc_field(
+                                { frameworkcode => $frameworkcode, tagfield => $tag->{fieldtag}, } );
                             $value = defined $authvalues->{$field->as_string} ? $authvalues->{$field->as_string} : $field->as_string;
                         } else {
                             # If it is a field, we gather all subfields, joined by the subfield separator
                             my @subvaluesarray;
                             my @subfields = $field->subfields;
                             foreach my $subfield (@subfields) {
+                                my $authvalues = Koha::AuthorisedValues->get_descriptions_by_marc_field(
+                                    {
+                                        frameworkcode => $frameworkcode, tagfield => $tag->{fieldtag},
+                                        tagsubfield   => $subfield->[0],
+                                    }
+                                );
                                 push (@subvaluesarray, defined $authvalues->{$subfield->[1]} ? $authvalues->{$subfield->[1]} : $subfield->[1]);
                             }
                             $value = join ($subfieldseparator, @subvaluesarray);
@@ -788,16 +794,7 @@ sub marc2bibtex {
         );
     }
 
-    my $BibtexExportAdditionalFields = C4::Context->preference('BibtexExportAdditionalFields');
-    my $additional_fields;
-    if ($BibtexExportAdditionalFields) {
-        $BibtexExportAdditionalFields = "$BibtexExportAdditionalFields\n\n";
-        $additional_fields = eval { YAML::XS::Load(Encode::encode_utf8($BibtexExportAdditionalFields)); };
-        if ($@) {
-            warn "Unable to parse BibtexExportAdditionalFields : $@";
-            $additional_fields = undef;
-        }
-    }
+    my $additional_fields = C4::Context->yaml_preference('BibtexExportAdditionalFields');
 
     if ( $additional_fields && $additional_fields->{'@'} ) {
         my ( $f, $sf ) = split( /\$/, $additional_fields->{'@'} );
