@@ -16,6 +16,7 @@ use C4::SIP::Sip::Checksum qw(verify_cksum);
 
 use Data::Dumper;
 use CGI qw ( -utf8 );
+use C4::Context;
 use C4::Auth qw(&check_api_auth);
 use C4::Items qw(ModDateLastSeen);
 
@@ -503,8 +504,12 @@ sub handle_patron_status {
 
     $ils->check_inst_id( $fields->{ (FID_INST_ID) }, "handle_patron_status" );
     $patron = $ils->find_patron( $fields->{ (FID_PATRON_ID) } );
+    if ( C4::Context->preference('TrackLastPatronActivityTriggers') && $patron ) {
+        my $koha_patron = Koha::Patrons->find($patron->{borrowernumber});
+        $koha_patron->update_lastseen('connection');
+    }
     $resp = build_patron_status( $patron, $lang, $fields, $server );
-    $self->write_msg( $resp, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+    $self->write_msg( $resp, $server );
     return (PATRON_STATUS_REQ);
 }
 
@@ -644,7 +649,7 @@ sub handle_checkout {
         }
     }
 
-    $self->write_msg( $resp, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+    $self->write_msg( $resp, $server );
     return (CHECKOUT);
 }
 
@@ -743,7 +748,7 @@ sub handle_checkin {
     $resp .= maybe_add( FID_SCREEN_MSG, $status->screen_msg, $server );
     $resp .= maybe_add( FID_PRINT_LINE, $status->print_line, $server );
 
-    $self->write_msg( $resp, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+    $self->write_msg( $resp, $server );
 
     return (CHECKIN);
 }
@@ -790,7 +795,7 @@ sub handle_block_patron {
     }
 
     $resp = build_patron_status( $patron, $patron->language, $fields, $server );
-    $self->write_msg( $resp, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+    $self->write_msg( $resp, $server );
     return (BLOCK_PATRON);
 }
 
@@ -831,7 +836,7 @@ sub handle_request_acs_resend {
 
         # We haven't sent anything yet, so respond with a
         # REQUEST_SC_RESEND msg (p. 16)
-        $self->write_msg( REQUEST_SC_RESEND, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+        $self->write_msg( REQUEST_SC_RESEND, $server );
     } elsif ( ( length($last_response) < 9 )
         || substr( $last_response, -9, 2 ) ne 'AY' ) {
 
@@ -845,7 +850,7 @@ sub handle_request_acs_resend {
         # Cut out the sequence number and checksum, since the old
         # checksum is wrong for the resent message.
         my $rebuilt = substr( $last_response, 0, -9 );
-        $self->write_msg( $rebuilt, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+        $self->write_msg( $rebuilt, $server );
     }
 
     return REQUEST_ACS_RESEND;
@@ -927,7 +932,7 @@ sub handle_login {
         $status = login_core( $server, $uid, $pwd );
     }
 
-    $self->write_msg( LOGIN_RESP . $status, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+    $self->write_msg( LOGIN_RESP . $status, $server );
     return $status ? LOGIN : '';
 }
 
@@ -966,7 +971,7 @@ sub summary_info {
     my $fid      = $summary_map[$summary_type]->{fid};
     my $itemlist = &$func( $patron, $start, $end, $server );
 
-    siplog( "LOG_DEBUG", "summary_info: list = (%s)", join( ", ", map{ $_->{barcode} } @{$itemlist} ) );
+    siplog( "LOG_DEBUG", "summary_info: list = (%s)", join( ", ", map { $_->{barcode} } @{$itemlist} ) );
     foreach my $i ( @{$itemlist} ) {
         $resp .= add_field( $fid, $i->{barcode}, $server );
     }
@@ -995,7 +1000,10 @@ sub handle_patron_info {
 
     $resp = (PATRON_INFO_RESP);
     if ($patron) {
-        $patron->update_lastseen();
+        if ( C4::Context->preference('TrackLastPatronActivityTriggers') ) {
+            my $koha_patron = Koha::Patrons->find($patron->{borrowernumber});
+            $koha_patron->update_lastseen('connection');
+        }
         $resp .= patron_status_string( $patron, $server );
         $resp .= ( defined($lang) and length($lang) == 3 ) ? $lang : $patron->language;
         $resp .= timestamp();
@@ -1093,7 +1101,7 @@ sub handle_patron_info {
         $resp .= maybe_add( FID_SCREEN_MSG, INVALID_CARD, $server );
     }
 
-    $self->write_msg( $resp, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+    $self->write_msg( $resp, $server );
     return (PATRON_INFO);
 }
 
@@ -1120,7 +1128,7 @@ sub handle_end_patron_session {
     $resp .= maybe_add( FID_SCREEN_MSG, $screen_msg, $server );
     $resp .= maybe_add( FID_PRINT_LINE, $print_line, $server );
 
-    $self->write_msg( $resp, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+    $self->write_msg( $resp, $server );
 
     return (END_PATRON_SESSION);
 }
@@ -1169,7 +1177,8 @@ sub handle_fee_paid {
         "on_reserve" => "On hold for another patron",
         "patron_restricted" => "Patron is currently restricted",
         "item_denied_renewal" => "Item is not allowed renewal",
-        "onsite_checkout" => "Item is an onsite checkout"
+        "onsite_checkout" => "Item is an onsite checkout",
+        "item_issued_to_other_patron" => "Item already issued to other borrower"
     };
     my @success = ();
     my @fail = ();
@@ -1201,7 +1210,7 @@ sub handle_fee_paid {
     $resp .= maybe_add( FID_SCREEN_MSG,     $status->screen_msg, $server );
     $resp .= maybe_add( FID_PRINT_LINE,     $status->print_line, $server );
 
-    $self->write_msg( $resp, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+    $self->write_msg( $resp, $server );
 
     return (FEE_PAID);
 }
@@ -1239,7 +1248,7 @@ sub handle_item_information {
         ModDateLastSeen( $item->itemnumber, $seen eq 'keep_lost' ) if $seen;
 
         # Valid Item ID, send the good stuff
-        my $circulation_status = $item->sip_circulation_status;
+        my $circulation_status = $item->sip_circulation_status($server);
         $resp .= $circulation_status;
         $resp .= $item->sip_security_marker;
         $resp .= $item->sip_fee_type;
@@ -1294,7 +1303,7 @@ sub handle_item_information {
         $resp .= $item->build_custom_field_string( $server );
     }
 
-    $self->write_msg( $resp, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+    $self->write_msg( $resp, $server );
 
     return (ITEM_INFORMATION);
 }
@@ -1344,7 +1353,7 @@ sub handle_item_status_update {
     $resp .= maybe_add( FID_SCREEN_MSG, $status->screen_msg, $server );
     $resp .= maybe_add( FID_PRINT_LINE, $status->print_line, $server );
 
-    $self->write_msg( $resp, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+    $self->write_msg( $resp, $server );
 
     return (ITEM_STATUS_UPDATE);
 }
@@ -1398,7 +1407,7 @@ sub handle_patron_enable {
 
     $resp .= add_field( FID_INST_ID, $ils->institution, $server );
 
-    $self->write_msg( $resp, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+    $self->write_msg( $resp, $server );
 
     return (PATRON_ENABLE);
 }
@@ -1463,7 +1472,7 @@ sub handle_hold {
     $resp .= maybe_add( FID_SCREEN_MSG, $status->screen_msg, $server );
     $resp .= maybe_add( FID_PRINT_LINE, $status->print_line, $server );
 
-    $self->write_msg( $resp, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+    $self->write_msg( $resp, $server );
 
     return (HOLD);
 }
@@ -1550,7 +1559,7 @@ sub handle_renew {
     $resp .= maybe_add( FID_SCREEN_MSG, $status->screen_msg, $server );
     $resp .= maybe_add( FID_PRINT_LINE, $status->print_line, $server );
 
-    $self->write_msg( $resp, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+    $self->write_msg( $resp, $server );
 
     return (RENEW);
 }
@@ -1597,13 +1606,13 @@ sub handle_renew_all {
     $resp .= timestamp;
     $resp .= add_field( FID_INST_ID, $ils->institution, $server );
 
-    $resp .= join( '', map( add_field( FID_RENEWED_ITEMS,   $_ ), @renewed ), $server );
-    $resp .= join( '', map( add_field( FID_UNRENEWED_ITEMS, $_ ), @unrenewed ), $server );
+    $resp .= join( '', map( add_field( FID_RENEWED_ITEMS,   $_, $server ), @renewed ) );
+    $resp .= join( '', map( add_field( FID_UNRENEWED_ITEMS, $_, $server ), @unrenewed ) );
 
     $resp .= maybe_add( FID_SCREEN_MSG, $status->screen_msg, $server );
     $resp .= maybe_add( FID_PRINT_LINE, $status->print_line, $server );
 
-    $self->write_msg( $resp, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+    $self->write_msg( $resp, $server );
 
     return (RENEW_ALL);
 }
@@ -1709,7 +1718,7 @@ sub send_acs_status {
 
     # Do we want to tell the terminal its location?
 
-    $self->write_msg( $msg, undef, $server->{account}->{terminator}, $server->{account}->{encoding} );
+    $self->write_msg( $msg, $server );
     return 1;
 }
 
@@ -1748,8 +1757,8 @@ sub api_auth {
     my ( $username, $password, $branch ) = @_;
     $ENV{REMOTE_USER} = $username;
     my $query = CGI->new();
-    $query->param( userid   => $username );
-    $query->param( password => $password );
+    $query->param( login_userid   => $username );
+    $query->param( login_password => $password );
     if ($branch) {
         $query->param( branch => $branch );
     }
