@@ -579,6 +579,7 @@ sub getRecords {
                             "type_label_"
                               . $facets_info->{$link_value}->{'label_value'} =>
                               1,
+                            label      => $facets_info->{$link_value}->{'label_value'},
                             facets     => \@this_facets_array,
                           }
                           unless (
@@ -1009,6 +1010,7 @@ sub getIndexes{
                     'Conference-name-seealso',
                     'Content-type',
                     'Control-number',
+                    'cnum',
                     'Control-number-identifier',
                     'cni',
                     'copydate',
@@ -1693,7 +1695,7 @@ sub searchResults {
         $times = $offset + $results_per_page;
     }
     else {
-        $times = $hits; # If less hits than results_per_page+offset we go to the end
+        $times = $hits // 0; # If less hits than results_per_page+offset we go to the end
     }
 
     my $marcflavour = C4::Context->preference("marcflavour");
@@ -1853,6 +1855,7 @@ sub searchResults {
                 ( $item->{$hbranch} ? $item->{$hbranch} . '--' : q{} )
               . ( $item->{location} ? $item->{location} : q{} )
               . ( $item->{itype}    ? $item->{itype}    : q{} )
+              . ( $item->{ccode}    ? $item->{ccode}    : q{} )
               . ( $item->{itemcallnumber} ? $item->{itemcallnumber} : q{} );
 # For each grouping of items (onloan, available, unavailable), we build a key to store relevant info about that item
             if ( $item->{onloan}
@@ -1869,6 +1872,7 @@ sub searchResults {
                 $onloan_items->{$key}->{description}    = $item->{description};
                 $onloan_items->{$key}->{imageurl} =
                   getitemtypeimagelocation( $search_context->{'interface'}, $itemtypes{ $item->{itype} }->{imageurl} );
+                $onloan_items->{$key}->{collectioncode} = GetAuthorisedValueDesc('','',$item->{ccode},'','','CCODE');
 
                 # if something's checked out and lost, mark it as 'long overdue'
                 if ( $item->{itemlost} ) {
@@ -1968,6 +1972,7 @@ sub searchResults {
                     $other_items->{$key}->{location} = $shelflocations->{ $item->{location} } if $item->{location};
                     $other_items->{$key}->{description} = $item->{description};
                     $other_items->{$key}->{imageurl} = getitemtypeimagelocation( $search_context->{'interface'}, $itemtypes{ $item->{itype}//q{} }->{imageurl} );
+                    $other_items->{$key}->{collectioncode} = GetAuthorisedValueDesc('','',$item->{ccode},'','','CCODE');
                 }
                 # item is available
                 else {
@@ -1978,6 +1983,7 @@ sub searchResults {
                     }
                     $available_items->{$prefix}->{location} = $shelflocations->{ $item->{location} } if $item->{location};
                     $available_items->{$prefix}->{imageurl} = getitemtypeimagelocation( $search_context->{'interface'}, $itemtypes{ $item->{itype}//q{} }->{imageurl} );
+                    $available_items->{$prefix}->{collectioncode} = GetAuthorisedValueDesc('','',$item->{ccode},'','','CCODE');
                 }
             }
         }    # notforloan, item level and biblioitem level
