@@ -23,13 +23,15 @@ use Carp;
 use Data::Dumper qw( Dumper );
 use Try::Tiny qw( catch try );
 
-use C4::Circulation qw( ReturnLostItem CanBookBeRenewed AddRenewal );
 use C4::Letters;
 use C4::Log qw( logaction );
 use C4::Stats qw( UpdateStats );
 use C4::Overdues qw(GetFine);
+use C4::Context;
 
 use Koha::Patrons;
+use Koha::Account::Credits;
+use Koha::Account::Debits;
 use Koha::Account::Lines;
 use Koha::Account::Offsets;
 use Koha::Account::DebitTypes;
@@ -168,13 +170,14 @@ my $credit_line = Koha::Account->new({ patron_id => $patron_id })->add_credit(
     {
         amount       => $amount,
         description  => $description,
-        note         => $note,
-        user_id      => $user_id,
         interface    => $interface,
+        issue_id     => $checkout->id,
+        item_id      => $item_id,
         library_id   => $library_id,
+        note         => $note,
         payment_type => $payment_type,
         type         => $credit_type,
-        item_id      => $item_id
+        user_id      => $user_id,
     }
 );
 
@@ -219,6 +222,7 @@ sub add_credit {
     my $payment_type  = $params->{payment_type};
     my $credit_type   = $params->{type} || 'PAYMENT';
     my $item_id       = $params->{item_id};
+    my $issue_id      = $params->{issue_id};
 
     Koha::Exceptions::Account::RegisterRequired->throw()
       if ( C4::Context->preference("UseCashRegisters")
@@ -248,6 +252,7 @@ sub add_credit {
                         branchcode        => $library_id,
                         register_id       => $cash_register,
                         itemnumber        => $item_id,
+                        issue_id          => $issue_id,
                     }
                 )->store();
 
@@ -266,6 +271,7 @@ sub add_credit {
                         type           => lc($credit_type),
                         amount         => $amount,
                         borrowernumber => $self->{patron_id},
+                        interface      => $interface,
                     }
                 ) if grep { $credit_type eq $_ } ( 'PAYMENT', 'WRITEOFF' );
 
@@ -712,31 +718,18 @@ my $non_issues_charges = $self->non_issues_charges
 
 Calculates amount immediately owing by the patron - non-issue charges.
 
-Charges exempt from non-issue are:
-* Res (holds) if HoldsInNoissuesCharge syspref is set to false
-* Rent (rental) if RentalsInNoissuesCharge syspref is set to false
-* Manual invoices if ManInvInNoissuesCharge syspref is set to false
+Charges can be set as exempt from non-issue by editing the debit type in the Debit Types area of System Preferences.
 
 =cut
 
 sub non_issues_charges {
     my ($self) = @_;
 
-    #NOTE: With bug 23049 these preferences could be moved to being attached
-    #to individual debit types to give more flexability and specificity.
-    my @not_fines;
-    push @not_fines, 'RESERVE'
-      unless C4::Context->preference('HoldsInNoissuesCharge');
-    push @not_fines, ( 'RENT', 'RENT_DAILY', 'RENT_RENEW', 'RENT_DAILY_RENEW' )
-      unless C4::Context->preference('RentalsInNoissuesCharge');
-    unless ( C4::Context->preference('ManInvInNoissuesCharge') ) {
-        my @man_inv = Koha::Account::DebitTypes->search({ is_system => 0 })->get_column('code');
-        push @not_fines, @man_inv;
-    }
+    my @blocking_debit_types = Koha::Account::DebitTypes->search({ restricts_checkouts => 1 }, { columns => 'code' })->get_column('code');
 
     return $self->lines->search(
         {
-            debit_type_code => { -not_in => \@not_fines }
+            debit_type_code => { -in => \@blocking_debit_types }
         },
     )->total_outstanding;
 }
@@ -759,6 +752,43 @@ sub lines {
     );
 }
 
+
+=head3 credits
+
+  my $credits = $self->credits;
+
+Return all credits for the user
+
+=cut
+
+sub credits {
+    my ($self) = @_;
+
+    return Koha::Account::Credits->search(
+        {
+            borrowernumber => $self->{patron_id}
+        }
+    );
+}
+
+=head3 debits
+
+  my $debits = $self->debits;
+
+Return all debits for the user
+
+=cut
+
+sub debits {
+    my ($self) = @_;
+
+    return Koha::Account::Debits->search(
+        {
+            borrowernumber   => $self->{patron_id},
+        }
+    );
+}
+
 =head3 reconcile_balance
 
 $account->reconcile_balance();
