@@ -20,7 +20,6 @@ use Modern::Perl;
 use Data::Dumper qw( Dumper );
 
 use C4::Log qw( logaction );
-use C4::Overdues qw( UpdateFine );
 
 use Koha::Account::CreditType;
 use Koha::Account::DebitType;
@@ -28,7 +27,7 @@ use Koha::Account::Offsets;
 use Koha::Database;
 use Koha::DateUtils qw( dt_from_string );
 use Koha::Exceptions::Account;
-use Koha::Items;
+use Koha::Patron::Debarments;
 
 use base qw(Koha::Object Koha::Object::Mixin::AdditionalFields);
 
@@ -52,7 +51,7 @@ Return the patron linked to this account line
 
 sub patron {
     my ( $self ) = @_;
-    my $rs = $self->_result->borrowernumber;
+    my $rs = $self->_result->patron;
     return unless $rs;
     return Koha::Patron->_new_from_dbic( $rs );
 }
@@ -80,6 +79,7 @@ sub item {
     my ( $self ) = @_;
     my $rs = $self->_result->itemnumber;
     return unless $rs;
+    require Koha::Item;
     return Koha::Item->_new_from_dbic( $rs );
 }
 
@@ -497,6 +497,10 @@ sub reduce {
         Koha::Exceptions::Account::IsNotDebit->throw(
             error => 'Account line ' . $self->id . 'is a payout' );
     }
+    if ( $self->debit_type_code eq 'VOID' ) {
+        Koha::Exceptions::Account::IsNotDebit->throw(
+            error => 'Account line ' . $self->id . 'is void' );
+    }
 
     # Check for mandatory parameters
     my @mandatory = ( 'interface', 'reduction_type', 'amount' );
@@ -632,73 +636,84 @@ sub apply {
 
     my $schema = Koha::Database->new->schema;
 
-    $schema->txn_do( sub {
-        for my $debit ( @{$debits} ) {
+    $schema->txn_do(
+        sub {
+            for my $debit ( @{$debits} ) {
 
-            unless ( $debit->is_debit ) {
-                Koha::Exceptions::Account::IsNotDebit->throw(
-                    error => 'Account line ' . $debit->id . 'is not a debit'
-                );
-            }
-            my $amount_to_cancel;
-            my $owed = $debit->amountoutstanding;
-
-            if ( $available_credit >= $owed ) {
-                $amount_to_cancel = $owed;
-            }
-            else {    # $available_credit < $debit->amountoutstanding
-                $amount_to_cancel = $available_credit;
-            }
-
-            # record the account offset
-            Koha::Account::Offset->new(
-                {   credit_id => $self->id,
-                    debit_id  => $debit->id,
-                    amount    => $amount_to_cancel * -1,
-                    type      => 'APPLY'
+                unless ( $debit->is_debit ) {
+                    Koha::Exceptions::Account::IsNotDebit->throw(
+                        error => 'Account line ' . $debit->id . 'is not a debit' );
                 }
-            )->store();
+                my $amount_to_cancel;
+                my $owed = $debit->amountoutstanding;
 
-            $available_credit -= $amount_to_cancel;
+                if ( $available_credit >= $owed ) {
+                    $amount_to_cancel = $owed;
+                } else {    # $available_credit < $debit->amountoutstanding
+                    $amount_to_cancel = $available_credit;
+                }
 
-            $self->amountoutstanding( $available_credit * -1 )->store;
-            $debit->amountoutstanding( $owed - $amount_to_cancel )->store;
-
-            # Attempt to renew the item associated with this debit if
-            # appropriate
-            if ( $self->credit_type_code ne 'FORGIVEN' && $debit->is_renewable ) {
-                my $outcome = $debit->renew_item( { interface => $params->{interface} } );
-                $self->add_message(
+                # record the account offset
+                Koha::Account::Offset->new(
                     {
-                        type    => 'info',
-                        message => 'renewal',
-                        payload => $outcome
+                        credit_id => $self->id,
+                        debit_id  => $debit->id,
+                        amount    => $amount_to_cancel * -1,
+                        type      => 'APPLY'
                     }
-                ) if $outcome;
-            }
-            $debit->discard_changes; # Refresh values from DB to clear floating point remainders
+                )->store();
 
-            # Same logic exists in Koha::Account::pay
-            if (
-                C4::Context->preference('MarkLostItemsAsReturned') =~
-                m|onpayment|
-                && $debit->debit_type_code
-                && $debit->debit_type_code eq 'LOST'
-                && $debit->amountoutstanding == 0
-                && $debit->itemnumber
-                && !(
-                       $self->credit_type_code eq 'LOST_FOUND'
-                    && $self->itemnumber == $debit->itemnumber
-                )
-              )
-            {
-                C4::Circulation::ReturnLostItem( $self->borrowernumber,
-                    $debit->itemnumber );
-            }
+                $available_credit -= $amount_to_cancel;
 
-            last if $available_credit == 0;
+                $self->amountoutstanding( $available_credit * -1 )->store;
+                $debit->amountoutstanding( $owed - $amount_to_cancel )->store;
+
+                # Attempt to renew the item associated with this debit if
+                # appropriate
+                if ( $self->credit_type_code ne 'FORGIVEN' && $debit->is_renewable ) {
+                    my $outcome = $debit->renew_item( { interface => $params->{interface} } );
+                    $self->add_message(
+                        {
+                            type    => 'info',
+                            message => 'renewal',
+                            payload => $outcome
+                        }
+                    ) if $outcome;
+                }
+                $debit->discard_changes;    # Refresh values from DB to clear floating point remainders
+
+                if (   $debit->debit_type_code
+                    && $debit->debit_type_code eq 'LOST'
+                    && $debit->amountoutstanding == 0
+                    && $debit->itemnumber
+                    && !( $self->credit_type_code eq 'LOST_FOUND' && $self->itemnumber == $debit->itemnumber ) )
+                {
+
+                    if ( C4::Context->preference('UpdateItemLostStatusWhenPaid')
+                        && !( $self->credit_type_code eq 'WRITEOFF' || $self->credit_type_code eq 'VOID' ) )
+                    {
+                        $debit->item->itemlost( C4::Context->preference('UpdateItemLostStatusWhenPaid') )->store();
+                    }
+
+                    if ( C4::Context->preference('UpdateItemLostStatusWhenWriteOff')
+                        && ( $self->credit_type_code eq 'WRITEOFF' || $self->credit_type_code eq 'VOID' ) )
+                    {
+                        $debit->item->itemlost( C4::Context->preference('UpdateItemLostStatusWhenWriteOff') )->store();
+                    }
+
+                    if ( C4::Context->preference('MarkLostItemsAsReturned') =~ m|onpayment| ) {
+                        C4::Circulation::ReturnLostItem(
+                            $self->borrowernumber,
+                            $debit->itemnumber
+                        );
+                    }
+                }
+                last if $available_credit == 0;
+            }
         }
-    });
+    );
+
+    Koha::Patron::Debarments::del_restrictions_after_payment( { borrowernumber => $self->borrowernumber } );
 
     return $self;
 }
@@ -954,7 +969,6 @@ on the API.
 sub to_api_mapping {
     return {
         accountlines_id   => 'account_line_id',
-        credit_number     => undef,
         credit_type_code  => 'credit_type',
         debit_type_code   => 'debit_type',
         amountoutstanding => 'amount_outstanding',
@@ -1022,20 +1036,19 @@ sub renew_item {
     }
 
     my $itemnumber = $self->item->itemnumber;
-    my $borrowernumber = $self->patron->borrowernumber;
     my ( $can_renew, $error ) = C4::Circulation::CanBookBeRenewed(
-        $borrowernumber,
-        $itemnumber
+        $self->patron,
+        $self->item->checkout
     );
     if ( $can_renew ) {
+        my $borrowernumber = $self->patron->borrowernumber;
         my $due_date = C4::Circulation::AddRenewal(
-            $borrowernumber,
-            $itemnumber,
-            $self->{branchcode},
-            undef,
-            undef,
-            undef,
-            0
+            {
+                borrowernumber => $borrowernumber,
+                itemnumber     => $itemnumber,
+                branch         => $self->{branchcode},
+                seen           => 0
+            }
         );
         return {
             itemnumber => $itemnumber,
