@@ -21,6 +21,7 @@ use Carp qw( croak );
 
 use C4::Biblio qw( DelBiblio );
 use C4::Acquisition;
+use C4::Suggestions qw( ModSuggestion );
 
 use Koha::Acquisition::Baskets;
 use Koha::Acquisition::Funds;
@@ -33,9 +34,10 @@ use Koha::Biblios;
 use Koha::Holds;
 use Koha::Items;
 use Koha::Number::Price;
+use Koha::Patrons;
 use Koha::Subscriptions;
 
-use base qw(Koha::Object);
+use base qw(Koha::Object Koha::Object::Mixin::AdditionalFields);
 
 =head1 NAME
 
@@ -143,11 +145,23 @@ sub cancel {
         }
     }
 
+    # If ordered from a suggestion, revert the suggestion status to ACCEPTED
+    my $suggestion = Koha::Suggestions->find({ biblionumber => $self->biblionumber, status => "ORDERED" });
+    if ( $suggestion and $suggestion->id ) {
+        ModSuggestion(
+            {
+                suggestionid => $suggestion->id,
+                biblionumber => $self->biblionumber,
+                STATUS       => 'ACCEPTED',
+            }
+        );
+    }
+
     my $biblio = $self->biblio;
     if ( $biblio and $delete_biblio ) {
 
         if (
-            $biblio->active_orders->search(
+            $biblio->uncancelled_orders->search(
                 { ordernumber => { '!=' => $self->ordernumber } }
             )->count == 0
             and $biblio->subscriptions->count == 0
@@ -168,10 +182,10 @@ sub cancel {
 
             my $message;
 
-            if ( $biblio->active_orders->search(
+            if ( $biblio->uncancelled_orders->search(
                 { ordernumber => { '!=' => $self->ordernumber } }
             )->count > 0 ) {
-                $message = 'error_delbiblio_active_orders';
+                $message = 'error_delbiblio_uncancelled_orders';
             }
             elsif ( $biblio->subscriptions->count > 0 ) {
                 $message = 'error_delbiblio_subscriptions';
@@ -189,12 +203,12 @@ sub cancel {
         }
     }
 
-    # Update order status
+    # Update order status; do not overwrite older cancellation date
+    $self->datecancellationprinted(dt_from_string) unless $self->datecancellationprinted;
     $self->set(
         {
-            cancellationreason      => $reason,
-            datecancellationprinted => \'NOW()',
-            orderstatus             => 'cancelled',
+            cancellationreason => $reason,
+            orderstatus        => 'cancelled',
         }
     )->store;
 
@@ -400,6 +414,21 @@ sub claimed_date {
     return $last_claim->claimed_on;
 }
 
+=head3 creator
+
+my $creator = $order->creator;
+
+Retrieves patron that created this order.
+
+=cut
+
+sub creator {
+    my ( $self )  = @_;
+    my $creator_rs = $self->_result->creator;
+    return unless $creator_rs;
+    return Koha::Patron->_new_from_dbic( $creator_rs );
+}
+
 =head3 duplicate_to
 
     my $duplicated_order = $order->duplicate_to($basket, [$default_values]);
@@ -426,7 +455,6 @@ sub duplicate_to {
                 invoiceid
                 datecancellationprinted
                 cancellationreason
-                purchaseordernumber
                 claims_count
                 claimed_date
                 parent_ordernumber
@@ -620,7 +648,6 @@ sub to_api_mapping {
         ordernumber                   => 'order_id',
         orderstatus                   => 'status',
         parent_ordernumber            => 'parent_order_id',
-        purchaseordernumber           => undef,                    # obsolete
         quantityreceived              => 'quantity_received',
         replacementprice              => 'replacement_price',
         sort1                         => 'statistics_1',
@@ -636,7 +663,8 @@ sub to_api_mapping {
         uncertainprice                => 'uncertain_price',
         unitprice                     => 'unit_price',
         unitprice_tax_excluded        => 'unit_price_tax_excluded',
-        unitprice_tax_included        => 'unit_price_tax_included'
+        unitprice_tax_included        => 'unit_price_tax_included',
+        invoice_unitprice             => 'invoice_unit_price',
     };
 }
 
