@@ -71,7 +71,7 @@ sub columns {
 
         aqorders => {
             "basketno"                      => __("Basket ID (internal)"),
-            "biblionumber"                  => __("Biblionumber (internal)"),
+            "biblionumber"                  => __("Biblio number (internal)"),
             "budget_id"                     => __("Fund ID (internal)"),
             "budgetdate"                    => __("Not used (deprecated)"),
             "cancellationreason"            => __("Cancellation reason"),
@@ -94,7 +94,6 @@ sub columns {
             "ordernumber"                   => __("Order number"),
             "orderstatus"                   => __("Order status"),
             "parent_ordernumber"            => __("Parent order number (internal)"),
-            "purchaseordernumber"           => __("Not used (deprecated)"),
             "quantity"                      => __("Quantity"),
             "quantityreceived"              => __("Quantity received"),
             "replacementprice"              => __("Replacement price"),
@@ -189,6 +188,7 @@ sub columns {
             "privacy_guarantor_fines"     => __("Show fines to guarantor"),
             "privacy"                     => __("Privacy settings"),
             "pronouns"                    => __("Pronouns"),
+            "protected"                   => __("Protected"),
             "relationship"                => __("Relationship"),
             "secret"                      => __("Secret (2FA)"),
             "sex"                         => __("Gender"),
@@ -206,68 +206,59 @@ sub columns {
             "zipcode"                     => __("ZIP/Postal code"),
         },
         items => {
-            "biblioitemnumber"                  => __("Biblioitem number (internal)"),
             "barcode"                           => __("Barcode"),
-            "dateaccessioned"                   => __("Date acquired"),
+            "biblioitemnumber"                  => __("Biblio item number (internal)"),
+            "biblionumber"                      => __("Biblio number (internal)"),
+            "bookable"                          => __("Bookable"),
             "booksellerid"                      => __("Source of acquisition"),
-            "homebranch"                        => __("Permanent library"),
-            "price"                             => __("Price"),
-            "replacementprice"                  => __("Replacement price"),
-            "replacementpricedate"              => __("Price effective from"),
-            "datelastborrowed"                  => __("Date last checked out"),
-            "datelastseen"                      => __("Date last seen"),
-            "stack"                             => __("Shelving control number"),
-            "notforloan"                        => __("Not for loan"),
+            "ccode"                             => __("Collection"),
+            "cn_sort"                           => __("Koha normalized classification for sorting"),
+            "cn_source"                         => __("Source of classification or shelving scheme"),
+            "coded_location_qualifier"          => __("Coded location qualifier"),
+            "copynumber"                        => __("Copy number"),
             "damaged"                           => __("Damaged status"),
             "damaged_on"                        => __("Damaged on"),
+            "dateaccessioned"                   => __("Date acquired"),
+            "datelastborrowed"                  => __("Date last checked out"),
+            "datelastseen"                      => __("Date last seen"),
+            "deleted_on"                        => __("Deleted on"),
+            "enumchron"                         => __("Serial enumeraton/chronology"),
+            "exclude_from_local_holds_priority" => __("Exclude from local holds priority"),
+            "holdingbranch"                     => __("Current library"),
+            "homebranch"                        => __("Permanent library"),
+            "issues"                            => __("Total checkouts"),
+            "itemcallnumber"                    => __("Call number"),
             "itemlost"                          => __("Lost status"),
             "itemlost_on"                       => __("Lost on"),
-            "withdrawn"                         => __("Withdrawn status"),
-            "withdrawn_on"                      => __("Withdrawn on"),
-            "itemcallnumber"                    => __("Call number"),
-            "coded_location_qualifier"          => __("Coded location qualifier"),
-            "issues"                            => __("Total checkouts"),
-            "renewals"                          => __("Total renewals"),
-            "reserves"                          => __("Total holds"),
-            "restricted"                        => __("Use restrictions"),
             "itemnotes"                         => __("Public note"),
             "itemnotes_nonpublic"               => __("Internal note"),
-            "holdingbranch"                     => __("Current library"),
-            "timestamp"                         => __("Timestamp"),
-            "deleted_on"                        => __("Deleted on"),
+            "itemnumber"                        => __("Koha item number (autogenerated)"),
+            "itype"                             => __("Koha item type"),
+            "localuse"                          => __("Total local uses"),
             "location"                          => __("Shelving location"),
-            "permanent_location"                => __("Permanent shelving location"),
-            "onloan"                            => __("Due date"),
-            "cn_source"                         => __("Source of classification / shelving scheme"),
-            "cn_sort"                           => __("Koha normalized classification for sorting"),
-            "ccode"                             => __("Collection"),
             "materials"                         => __("Materials specified"),
-            "uri"                               => __("Uniform Resource Identifier"),
-            "itype"                             => __("Koha itemtype"),
             "more_subfields_xml"                => __("Additional subfields (XML)"),
-            "enumchron"                         => __("Serial enumeraton/chronology"),
-            "copynumber"                        => __("Copy number"),
-            "stocknumber"                       => __("Inventory number"),
             "new_status"                        => __("New status"),
-            "exclude_from_local_holds_priority" => __("Exclude from local holds priority"),
-        },
-        statistics => {
-            "datetime"       => __("Statistics date and time"),
-            "branch"         => __("Library"),
-            "value"          => __("Value"),
-            "type"           => __("Transaction type"),
-            "other"          => __("SIP mode"),
-            "itemnumber"     => __("Item number"),
-            "itemtype"       => __("Koha item type"),
-            "location"       => __("Shelving location"),
-            "borrowernumber" => __("Borrower number"),
-            "ccode"          => __("Collection"),
-            "categorycode"   => __("Patron category"),
+            "notforloan"                        => __("Not for loan"),
+            "onloan"                            => __("Due date"),
+            "permanent_location"                => __("Permanent shelving location"),
+            "price"                             => __("Price"),
+            "renewals"                          => __("Total renewals"),
+            "replacementprice"                  => __("Replacement price"),
+            "replacementpricedate"              => __("Price effective from"),
+            "reserves"                          => __("Total holds"),
+            "restricted"                        => __("Use restrictions"),
+            "stack"                             => __("Shelving control number"),
+            "stocknumber"                       => __("Inventory number"),
+            "timestamp"                         => __("Modification date"),
+            "uri"                               => __("Uniform Resource Identifier"),
+            "withdrawn"                         => __("Withdrawn status"),
+            "withdrawn_on"                      => __("Withdrawn on"),
         },
         biblio => {
             "abstract"      => __("Abstract"),
             "author"        => __("Author"),
-            "biblionumber"  => __("Biblio number"),
+            "biblionumber"  => __("Biblio number (internal)"),
             "copyrightdate" => __("Copyright date"),
             "datecreated"   => __("Creation date"),
             "frameworkcode" => __("Framework code"),
@@ -283,39 +274,51 @@ sub columns {
             "unititle"      => __("Uniform title"),
         },
         biblioitems => {
-            "agerestriction"   => __("Age restriction"),
-            "biblioitemnumber" => __("Biblioitem number"),
-            "biblionumber"     => __("Biblio number"),
-            "classification"   => __("Classification"),
-            "dewey"            => __("Dewey/classification"),
-            "illus"            => __("Illustrations"),
-            "isbn"             => __("ISBN"),
-            "issn"             => __("ISSN"),
-            "itemtype"         => __("Biblio-level item type"),
-            "lccn"             => __("LCCN"),
-            "notes"            => __("Notes"),
-            "number"           => __("Number"),
-            "pages"            => __("Number of pages"),
-            "place"            => __("Place of publication"),
-            "publicationyear"  => __("Publication date"),
-            "publishercode"    => __("Publisher"),
-            "size"             => __("Size"),
-            "subclass"         => __("Sub classification"),
-            "timestamp"        => __("Timestamp"),
-            "title"            => __("Title"),
-            "url"              => __("URL"),
-            "volume"           => __("Volume number"),
-            "volumedate"       => __("Volume date"),
-            "volumedesc"       => __("Volume information"),
+            "agerestriction"        => __("Age restriction"),
+            "biblioitemnumber"      => __("Biblio item number (internal)"),
+            "biblionumber"          => __("Biblio number (internal)"),
+            "cn_class"              => __("Classification part"),
+            "cn_item"               => __("Item part"),
+            "cn_sort"               => __("Koha normalized classification for sorting"),
+            "cn_source"             => __("Source of classification or shelving scheme"),
+            "cn_suffix"             => __("Call number suffix"),
+            "collectiontitle"       => __("Series statement"),
+            "collectionissn"        => __("Series ISSN"),
+            "collectionvolume"      => __("Series volume"),
+            "ean"                   => __("EAN"),
+            "editionresponsibility" => __("Edition responsibility"),
+            "editionstatement"      => __("Edition statement"),
+            "illus"                 => __("Other physical details"),
+            "isbn"                  => __("ISBN"),
+            "issn"                  => __("ISSN"),
+            "itemtype"              => __("Biblio-level item type"),
+            "lccn"                  => __("LCCN"),
+            "notes"                 => __("Notes"),
+            "number"                => __("Number"),
+            "pages"                 => __("Number of pages"),
+            "place"                 => __("Place of publication"),
+            "publicationyear"       => __("Publication date"),
+            "publishercode"         => __("Publisher"),
+            "size"                  => __("Size"),
+            "timestamp"             => __("Modification date"),
+            "totalissues"           => __("Koha issues (borrowed), all copies"),
+            "url"                   => __("URL"),
+            "volume"                => __("Volume number"),
+            "volumedate"            => __("Volume date"),
+            "volumedesc"            => __("Volume information"),
         },
         statistics => {
             "borrowernumber" => __("Borrower number"),
             "branch"         => __("Library"),
+            "categorycode"   => __("Patron category"),
+            "ccode"          => __("Collection"),
             "datetime"       => __("Statistics date and time"),
+            "interface"      => __("Interface"),
             "itemnumber"     => __("Item number"),
-            "itemtype"       => __("Itemtype"),
-            "other"          => __(""),
-            "type"           => __("Type"),
+            "itemtype"       => __("Koha item type"),
+            "location"       => __("Shelving location"),
+            "other"          => __("SIP mode"),
+            "type"           => __("Transaction type"),
             "value"          => __("Value"),
         },
         subscription => {
@@ -332,7 +335,7 @@ sub columns {
             "collectiontitle" => __("Collection title"),
             "copyrightdate"   => __("Copyright date"),
             "isbn"            => __("ISBN"),
-            "itemtype"        => __("Item type"),
+            "itemtype"        => __("Koha item type"),
             "note"            => __("Note"),
             "patronreason"    => __("Patron reason"),
             "place"           => __("Place of publication"),
