@@ -45,6 +45,7 @@ use Koha::Plugins; # Adds plugin dirs to @INC
 use Koha::Plugins::Handler;
 use Koha::Acquisition::Baskets;
 use Koha::Acquisition::Booksellers;
+use Koha::Util::FrameworkPlugin qw( biblio_008 );
 
 our $VERSION = 1.1;
 
@@ -476,8 +477,7 @@ sub receipt_items {
             my $order = Koha::Acquisition::Orders->find($ordernumber);
             $biblionumber = $order->biblionumber;
             my $frameworkcode = GetFrameworkCode($biblionumber);
-            ($itemfield) = GetMarcFromKohaField( 'items.itemnumber',
-                $frameworkcode );
+            ($itemfield) = GetMarcFromKohaField('items.itemnumber');
         }
     }
 
@@ -685,6 +685,10 @@ sub quote_item {
     if ( !defined $bib ) {
         $bib = {};
         my $bib_record = _create_bib_from_quote( $item, $quote );
+
+        # Check for and add default 008 as this is a mandatory field
+        $bib_record = _handle_008_field($bib_record);
+
         ( $bib->{biblionumber}, $bib->{biblioitemnumber} ) =
           AddBiblio( $bib_record, q{} );
         $logger->trace("New biblio added $bib->{biblionumber}");
@@ -829,6 +833,20 @@ sub quote_item {
                     }
                 );
                 ++$created;
+
+                my $lrp = $item->girfield('library_rotation_plan');
+                if ($lrp) {
+                    my $rota = Koha::StockRotationRotas->find(
+                        { title => $lrp },
+                        { key   => 'stockrotationrotas_title' }
+                    );
+                    if ($rota) {
+                        $rota->add_item($itemnumber);
+                        $logger->trace("Item added to rota $rota->id");
+                    } else {
+                        $logger->error("No rota found matching $lrp in orderline");
+                    }
+                }
             }
         }
     }
@@ -1202,6 +1220,17 @@ sub _create_item_from_quote {
     return $item_hash;
 }
 
+sub _handle_008_field {
+    my ($bib_record) = @_;
+
+    if ( !$bib_record->field('008') ) {
+        my $default008 = biblio_008();
+        $bib_record->insert_fields_ordered( MARC::Field->new( '008', $default008 ) );
+    }
+
+    return $bib_record;
+}
+
 1;
 __END__
 
@@ -1360,6 +1389,14 @@ Koha::EDI
      Returns the Aqbudget object for the active budget given the passed budget_code
      or undefined if one does not exist
 
+=head2 _handle_008_field
+
+     bib_record = _handle_008_field($bib_record)
+
+     Checks whether an 008 field exists on the record and adds a default field it does not
+
+     Returns the bib_record
+
 =head1 AUTHOR
 
    Colin Campbell <colin.campbell@ptfs-europe.com>
