@@ -51,7 +51,10 @@ Koha::Items - Koha Item object set class
 
     my $filtered_items = $items->filter_by_for_hold;
 
-Return the items of the set that are *potentially* holdable.
+Return the items of the set that are *potentially* holdable. This routine
+checks only rules defined in the 'Standard rules for all libraries' and
+should only be used in the context where we don't have a patron to check
+policies for specifically.
 
 Caller has the responsibility to call C4::Reserves::CanItemBeReserved before
 placing a hold on one of those items.
@@ -61,15 +64,40 @@ placing a hold on one of those items.
 sub filter_by_for_hold {
     my ($self) = @_;
 
-    my @hold_not_allowed_itypes = Koha::CirculationRules->search(
+    my $default_rule = Koha::CirculationRules->get_effective_rule(
         {
             rule_name    => 'holdallowed',
-            branchcode   => undef,
-            categorycode => undef,
-            rule_value   => 'not_allowed',
         }
-    )->get_column('itemtype');
+    );
+    my @hold_not_allowed_itypes;
+    if ( defined $default_rule && $default_rule->rule_value eq 'not_allowed' ) {
+        # If the default rule is not allowed we get all itemtypes as not allowed
+        @hold_not_allowed_itypes = Koha::ItemTypes->search->get_column('itemtype');
+        my @hold_allowed_itypes = Koha::CirculationRules->search(
+            {
+                rule_name    => 'holdallowed',
+                rule_value   => { '!=' => 'not_allowed' },
+                branchcode   => undef,
+                categorycode => undef,
+            }
+        )->get_column('itemtype');
+        # We then only allow those explicitly defined in hold policies at the all libraries level
+        @hold_not_allowed_itypes = array_minus( @hold_not_allowed_itypes, @hold_allowed_itypes );
+    } else {
+        # If there is no default 'not_allowed' rule, then only those explicitly forbidden at the all libraries level are forbidden
+        @hold_not_allowed_itypes = Koha::CirculationRules->search(
+            {
+                rule_name    => 'holdallowed',
+                branchcode   => undef,
+                categorycode => undef,
+                rule_value   => 'not_allowed',
+            }
+        )->get_column('itemtype');
+    }
+
+    # We also forbid holds on any marked not for loan at the item level
     push @hold_not_allowed_itypes, Koha::ItemTypes->search({ notforloan => 1 })->get_column('itemtype');
+    @hold_not_allowed_itypes = uniq @hold_not_allowed_itypes;
 
     my $params = {
         itemlost   => 0,
@@ -158,6 +186,21 @@ sub filter_out_lost {
     return $self->search( $params );
 }
 
+=head3 filter_by_bookable
+
+  my $filterd_items = $items->filter_by_bookable;
+
+Returns a new resultset, containing only those items that are allowed to be booked.
+
+=cut
+
+sub filter_by_bookable {
+    my ($self) = @_;
+
+    my $params = { bookable => 1 };
+
+    return $self->search($params);
+}
 
 =head3 move_to_biblio
 
@@ -227,6 +270,11 @@ Allows to modify existing subfield's values using a regular expression
 
 Set the passed boolean value to items.exclude_from_local_holds_priority
 
+=item mark_items_returned
+
+Move issues on these items to the old issues table, do not mark items found, or
+adjust damaged/withdrawn statuses, or fines, or locations.
+
 =item callback
 
 Callback function to call after an item has been modified
@@ -241,6 +289,7 @@ sub batch_update {
     my $regex_mod = $params->{regex_mod} || {};
     my $new_values = $params->{new_values} || {};
     my $exclude_from_local_holds_priority = $params->{exclude_from_local_holds_priority};
+    my $mark_items_returned = $params->{mark_items_returned};
     my $callback = $params->{callback};
 
     my (@modified_itemnumbers, $modified_fields);
@@ -250,6 +299,7 @@ sub batch_update {
 
         try {$schema->txn_do(sub {
             my $modified_holds_priority = 0;
+            my $item_returned = 0;
             if ( defined $exclude_from_local_holds_priority ) {
                 if(!defined $item->exclude_from_local_holds_priority || $item->exclude_from_local_holds_priority != $exclude_from_local_holds_priority) {
                     $item->exclude_from_local_holds_priority($exclude_from_local_holds_priority)->store;
@@ -336,9 +386,25 @@ sub batch_update {
                 ) if $item->itemlost
                       and not $itemlost_pre;
             }
+            if ( $mark_items_returned ){
+                my $issue = $item->checkout;
+                if( $issue ){
+                        $item_returned = 1;
+                        C4::Circulation::MarkIssueReturned(
+                        $issue->borrowernumber,
+                        $item->itemnumber,
+                        undef,
+                        $issue->patron->privacy,
+                        {
+                            skip_record_index => 1,
+                            skip_holds_queue  => 1,
+                        }
+                    );
+                }
+            }
 
-            push @modified_itemnumbers, $item->itemnumber if $modified || $modified_holds_priority;
-            $modified_fields += $modified + $modified_holds_priority;
+            push @modified_itemnumbers, $item->itemnumber if $modified || $modified_holds_priority || $item_returned;
+            $modified_fields += $modified + $modified_holds_priority + $item_returned;
         })}
         catch {
             warn $_
@@ -412,7 +478,7 @@ sub search_ordered {
 
     $self = $self->search($params, $attributes);
 
-    my @biblionumbers = uniq $self->get_column('biblionumber');
+    my @biblionumbers = uniq $self->search(undef,{distinct=>1})->get_column('biblionumber');
 
     if ( scalar ( @biblionumbers ) == 1
         && Koha::Biblios->find( $biblionumbers[0] )->serial )
@@ -431,7 +497,6 @@ sub search_ordered {
                 order_by => [
                     'homebranch.branchname',
                     'me.enumchron',
-                    \"LPAD( me.copynumber, 8, '0' )",
                     {-desc => 'me.dateaccessioned'}
                 ],
                 join => ['homebranch']
