@@ -34,6 +34,22 @@ Koha::Patron::Messages - Koha Message Object set class
 
 =cut
 
+=head3 filter_by_unread
+
+    Returns a resultset of messages that have not been marked read by the patron
+
+=cut
+
+sub filter_by_unread {
+    my ( $self ) = @_;
+
+    return $self->search(
+        {
+            patron_read_date => { is => undef },
+        }
+    );
+}
+
 =head3 _type
 
 =cut
@@ -42,6 +58,10 @@ sub _type {
     return 'Message';
 }
 
+=head3 object_class
+
+=cut
+
 sub object_class {
     return 'Koha::Patron::Message';
 }
