@@ -35,7 +35,8 @@ use Try::Tiny qw( catch try );
 sub list {
     my $c = shift->openapi->valid_input or return;
 
-    my $max_expiration_date = delete $c->validation->output->{max_expiration_date};
+    my $max_expiration_date = $c->param('max_expiration_date');
+    $c->req->params->remove('max_expiration_date');
 
     return try {
         my $agreements_set = Koha::ERM::Agreements->new;
@@ -61,15 +62,10 @@ sub get {
     my $c = shift->openapi->valid_input or return;
 
     return try {
-        my $agreement_id = $c->validation->param('agreement_id');
-        my $agreement    = $c->objects->find( Koha::ERM::Agreements->search, $agreement_id );
+        my $agreement = $c->objects->find( Koha::ERM::Agreements->search, $c->param('agreement_id') );
 
-        unless ($agreement) {
-            return $c->render(
-                status  => 404,
-                openapi => { error => "Agreement not found" }
-            );
-        }
+        return $c->render_resource_not_found("Agreement")
+            unless $agreement;
 
         return $c->render(
             status  => 200,
@@ -94,7 +90,7 @@ sub add {
         Koha::Database->new->schema->txn_do(
             sub {
 
-                my $body = $c->validation->param('body');
+                my $body = $c->req->json;
 
                 my $periods    = delete $body->{periods} // [];
                 my $user_roles = delete $body->{user_roles} // [];
@@ -112,7 +108,7 @@ sub add {
                 $c->res->headers->location($c->req->url->to_string . '/' . $agreement->agreement_id);
                 return $c->render(
                     status  => 201,
-                    openapi => $agreement->to_api
+                    openapi => $c->objects->to_api($agreement),
                 );
             }
         );
@@ -169,21 +165,16 @@ Controller function that handles updating a Koha::ERM::Agreement object
 sub update {
     my $c = shift->openapi->valid_input or return;
 
-    my $agreement_id = $c->validation->param('agreement_id');
-    my $agreement = Koha::ERM::Agreements->find( $agreement_id );
+    my $agreement = Koha::ERM::Agreements->find( $c->param('agreement_id') );
 
-    unless ($agreement) {
-        return $c->render(
-            status  => 404,
-            openapi => { error => "Agreement not found" }
-        );
-    }
+    return $c->render_resource_not_found("Agreement")
+        unless $agreement;
 
     return try {
         Koha::Database->new->schema->txn_do(
             sub {
 
-                my $body = $c->validation->param('body');
+                my $body = $c->req->json;
 
                 my $periods    = delete $body->{periods} // [];
                 my $user_roles = delete $body->{user_roles} // [];
@@ -201,7 +192,7 @@ sub update {
                 $c->res->headers->location($c->req->url->to_string . '/' . $agreement->agreement_id);
                 return $c->render(
                     status  => 200,
-                    openapi => $agreement->to_api
+                    openapi => $c->objects->to_api($agreement),
                 );
             }
         );
@@ -249,22 +240,15 @@ sub update {
 sub delete {
     my $c = shift->openapi->valid_input or return;
 
-    my $agreement = Koha::ERM::Agreements->find( $c->validation->param('agreement_id') );
-    unless ($agreement) {
-        return $c->render(
-            status  => 404,
-            openapi => { error => "Agreement not found" }
-        );
-    }
+    my $agreement = Koha::ERM::Agreements->find( $c->param('agreement_id') );
+
+    return $c->render_resource_not_found("Agreement")
+        unless $agreement;
 
     return try {
         $agreement->delete;
-        return $c->render(
-            status  => 204,
-            openapi => q{}
-        );
-    }
-    catch {
+        return $c->render_resource_deleted;
+    } catch {
         $c->unhandled_exception($_);
     };
 }
