@@ -50,7 +50,7 @@ use Koha::Biblio::ItemGroup::Items;
 use Koha::Biblio::ItemGroups;
 use Koha::CoverImages;
 use Koha::DateUtils;
-use Koha::Illrequests;
+use Koha::ILL::Requests;
 use Koha::Items;
 use Koha::ItemTypes;
 use Koha::Patrons;
@@ -190,24 +190,13 @@ $template->param(
 );
 
 my $itemtypes = { map { $_->itemtype => $_ } @{ Koha::ItemTypes->search_with_localization->as_list } };
-my $all_items = $biblio->items->search_ordered;
-my @items;
 my $patron = Koha::Patrons->find( $borrowernumber );
-while ( my $item = $all_items->next ) {
-    push @items, $item
-      unless $item->itemlost
-      && $patron->category->hidelostitems
-      && !$showallitems;
-}
-
-# flag indicating existence of at least one item linked via a host record
-my $hostrecords;
-# adding items linked via host biblios
-my $hostitems = $biblio->host_items;
-if ( $hostitems->count ) {
-    $hostrecords = 1;
-    push @items, $hostitems->as_list;
-}
+my $include_lost_items = !$patron->category->hidelostitems || $showallitems;
+my $items_params = {
+    ( $invalid_marc_record ? () : ( host_items => 1 ) ),
+};
+my $all_items = $biblio->items($items_params);
+my $items_to_display = $all_items->search({ $include_lost_items ? () : ( itemlost => 0 ) });
 
 my $dat = &GetBiblioData($biblionumber);
 
@@ -274,8 +263,14 @@ if ( $showcomp eq 'both' || $showcomp eq 'staff' ) {
     $template->param( analytics_error => 1 ) if grep { $_->message eq 'component_search' } @{$biblio->object_messages};
 }
 
+# Display volumes link
+my $show_volumes = ( !$invalid_marc_record && @{ $biblio->get_marc_volumes(1) } ) ? 1 : 0;
+
 # XSLT processing of some stuff
-my $xslt_variables = { show_analytics_link => $show_analytics };
+my $xslt_variables = {
+    show_analytics_link => $show_analytics,
+    show_volumes_link   => $show_volumes
+};
 $template->param(
     XSLTDetailsDisplay => '1',
     XSLTBloc => XSLTParse4Display({
@@ -298,7 +293,8 @@ if ( C4::Context->preference('AcquisitionDetails') ) {
     );    # GetHistory sorted by aqbooksellerid, but does it make sense?
 
     $template->param(
-        orders => $orders,
+        orders     => $orders,
+        acq_status => $biblio->acq_status,
     );
 }
 
@@ -320,186 +316,18 @@ if ( defined $dat->{'itemtype'} ) {
     $dat->{imageurl} = getitemtypeimagelocation( 'intranet', $itemtypes->{ $dat->{itemtype} }->imageurl );
 }
 
-$dat->{'count'} = $all_items->count + $hostitems->count;
-$dat->{'showncount'} = scalar @items + $hostitems->count;
-$dat->{'hiddencount'} = $all_items->count + $hostitems->count - scalar @items;
-
-my $shelflocations =
-  { map { $_->{authorised_value} => $_->{lib} } Koha::AuthorisedValues->get_descriptions_by_koha_field( { frameworkcode => $fw, kohafield => 'items.location' } ) };
-my $collections =
-  { map { $_->{authorised_value} => $_->{lib} } Koha::AuthorisedValues->get_descriptions_by_koha_field( { frameworkcode => $fw, kohafield => 'items.ccode' } ) };
-my $copynumbers =
-  { map { $_->{authorised_value} => $_->{lib} } Koha::AuthorisedValues->get_descriptions_by_koha_field( { frameworkcode => $fw, kohafield => 'items.copynumber' } ) };
-my (@itemloop, @otheritemloop, %itemfields);
-
-my $mss = Koha::MarcSubfieldStructures->search({ frameworkcode => $fw, kohafield => 'items.itemlost', authorised_value => [ -and => {'!=' => undef }, {'!=' => ''}] });
-if ( $mss->count ) {
-    $template->param( itemlostloop => GetAuthorisedValues( $mss->next->authorised_value ) );
+if ( C4::Context->preference('SeparateHoldings') ) {
+    my $SeparateHoldingsBranch = C4::Context->preference('SeparateHoldingsBranch') || 'homebranch';
+    my $other_holdings_count = $items_to_display->search({ $SeparateHoldingsBranch => { '!=' => C4::Context->userenv->{branch} } })->count;
+    $template->param( other_holdings_count => $other_holdings_count );
 }
-$mss = Koha::MarcSubfieldStructures->search({ frameworkcode => $fw, kohafield => 'items.damaged', authorised_value => [ -and => {'!=' => undef }, {'!=' => ''}] });
-if ( $mss->count ) {
-    $template->param( itemdamagedloop => GetAuthorisedValues( $mss->next->authorised_value ) );
-}
-$mss = Koha::MarcSubfieldStructures->search({ frameworkcode => $fw, kohafield => 'items.withdrawn', authorised_value => { not => undef } });
-if ( $mss->count ) {
-    $template->param( itemwithdrawnloop => GetAuthorisedValues( $mss->next->authorised_value) );
-}
-
-$mss = Koha::MarcSubfieldStructures->search({ frameworkcode => $fw, kohafield => 'items.materials', authorised_value => [ -and => {'!=' => undef }, {'!=' => ''}] });
-my %materials_map;
-if ($mss->count) {
-    my $materials_authvals = GetAuthorisedValues($mss->next->authorised_value);
-    if ($materials_authvals) {
-        foreach my $value (@$materials_authvals) {
-            $materials_map{$value->{authorised_value}} = $value->{lib};
-        }
-    }
-}
-
-my $analytics_flag;
-my $materials_flag; # set this if the items have anything in the materials field
-my $currentbranch = C4::Context->userenv ? C4::Context->userenv->{branch} : undef;
-if ($currentbranch and C4::Context->preference('SeparateHoldings')) {
-    $template->param(SeparateHoldings => 1);
-}
-my $separatebranch = C4::Context->preference('SeparateHoldingsBranch') || 'homebranch';
-my ( $itemloop_has_images, $otheritemloop_has_images );
-
-foreach my $item (@items) {
-    my $itembranchcode = $item->$separatebranch;
-
-    my $item_info = $item->unblessed;
-    $item_info->{itemtype} = $itemtypes->{$item->effective_itemtype};
-
-    #get shelf location and collection code description if they are authorised value.
-    # same thing for copy number
-    my $shelfcode = $item->location;
-    $item_info->{'location'} = $shelflocations->{$shelfcode} if ( defined( $shelfcode ) && defined($shelflocations) && exists( $shelflocations->{$shelfcode} ) );
-    my $ccode = $item->ccode;
-    $item_info->{'ccode'} = $collections->{$ccode} if ( defined( $ccode ) && defined($collections) && exists( $collections->{$ccode} ) );
-    my $copynumber = $item->copynumber;
-    $item_info->{'copynumber'} = $copynumbers->{$copynumber} if ( defined($copynumber) && defined($copynumbers) && exists( $copynumbers->{$copynumber} ) );
-    foreach (qw(ccode enumchron copynumber stocknumber itemnotes itemnotes_nonpublic uri )) {
-        $itemfields{$_} = 1 if $item->$_;
-    }
-
-    # FIXME The following must be Koha::Item->serial
-    my $serial_item = Koha::Serial::Items->find($item->itemnumber);
-    if ( $serial_item ) {
-        my $serial = Koha::Serials->find($serial_item->serialid);
-        $item_info->{serial} = $serial if $serial;
-        $itemfields{publisheddate} = 1;
-    }
-
-    $item_info->{object} = $item;
-
-    # checking for holds
-    my $holds = $item->current_holds;
-    if ( my $first_hold = $holds->next ) {
-        $item_info->{first_hold} = $first_hold;
-    }
-
-    $item_info->{checkout} = $item->checkout;
-
-    # Check the transit status
-    my $transfer = $item->get_transfer;
-    if ( $transfer ) {
-        $item_info->{transfer} = $transfer;
-    }
-
-    foreach my $f (qw( itemnotes )) {
-        if ($item_info->{$f}) {
-            $item_info->{$f} =~ s|\n|<br />|g;
-            $itemfields{$f} = 1;
-        }
-    }
-
-    #item has a host number if its biblio number does not match the current bib
-
-    if ($item->biblionumber ne $biblionumber){
-        $item_info->{hostbiblionumber} = $item->biblionumber;
-        $item_info->{hosttitle} = $item->biblio->title;
-    }
-
-
-    if ( $analyze ) {
-        # count if item is used in analytical bibliorecords
-        # The 'countanalytics' flag is only used in the templates if analyze is set
-        my $countanalytics = GetAnalyticsCount( $item->itemnumber );
-        if ($countanalytics > 0){
-            $analytics_flag=1;
-            $item_info->{countanalytics} = $countanalytics;
-        }
-    }
-
-    if (defined($item->materials) && $item->materials =~ /\S/){
-        $materials_flag = 1;
-        if (defined $materials_map{ $item->materials }) {
-            $item_info->{materials} = $materials_map{ $item->materials };
-        }
-    }
-
-    if ( C4::Context->preference('UseCourseReserves') ) {
-        $item_info->{'course_reserves'} = GetItemCourseReservesInfo( itemnumber => $item->itemnumber );
-    }
-
-    if ( C4::Context->preference("LocalCoverImages") == 1 ) {
-        $item_info->{cover_images} = $item->cover_images;
-    }
-
-    if ( C4::Context->preference('UseRecalls') ) {
-        $item_info->{recall} = $item->recall;
-    }
-
-    if ( C4::Context->preference('IndependentBranches') ) {
-        my $userenv = C4::Context->userenv();
-        if ( not C4::Context->IsSuperLibrarian()
-            and $userenv->{branch} ne $item->homebranch ) {
-            $item_info->{cannot_be_edited} = 1;
-            $item_info->{not_same_branch} = 1;
-        }
-    }
-
-    if ( $item->is_bundle ) {
-        $item_info->{bundled} =
-          $item->bundle_items->search( { itemlost => { '!=' => 0 } } )
-          ->count;
-        $item_info->{bundled_lost} =
-          $item->bundle_items->search( { itemlost => 0 } )->count;
-        $item_info->{is_bundle} = 1;
-    }
-
-    if ($item->in_bundle) {
-        $item_info->{bundle_host} = $item->bundle_host;
-    }
-
-    if ($currentbranch and C4::Context->preference('SeparateHoldings')) {
-        if ($itembranchcode and $itembranchcode eq $currentbranch) {
-            push @itemloop, $item_info;
-            $itemloop_has_images++ if $item->cover_images->count;
-        } else {
-            push @otheritemloop, $item_info;
-            $otheritemloop_has_images++ if $item->cover_images->count;
-        }
-    } else {
-        push @itemloop, $item_info;
-        $itemloop_has_images++ if $item->cover_images->count;
-    }
-}
-
 $template->param(
-    itemloop_has_images      => $itemloop_has_images,
-    otheritemloop_has_images => $otheritemloop_has_images,
+    count => $all_items->count, # FIXME 'count' is used in catalog-strings.inc
+                                # But it's not a meaningful variable, we should rename it there
+    all_items_count => $all_items->count,
+    items_to_display_count => $items_to_display->count,
 );
 
-# Display only one tab if one items list is empty
-if (scalar(@itemloop) == 0 || scalar(@otheritemloop) == 0) {
-    $template->param(SeparateHoldings => 0);
-    if (scalar(@itemloop) == 0) {
-        @itemloop = @otheritemloop;
-    }
-}
-
 my $some_private_shelves = Koha::Virtualshelves->get_some_shelves(
     {
         borrowernumber => $borrowernumber,
@@ -523,23 +351,11 @@ $template->param(
 
 $template->param(
     MARCNOTES               => !$invalid_marc_record ? $biblio->get_marc_notes() : undef,
-    itemdata_ccode          => $itemfields{ccode},
-    itemdata_enumchron      => $itemfields{enumchron},
-    itemdata_uri            => $itemfields{uri},
-    itemdata_copynumber     => $itemfields{copynumber},
-    itemdata_stocknumber    => $itemfields{stocknumber},
-    itemdata_publisheddate  => $itemfields{publisheddate},
-    volinfo                 => $itemfields{enumchron},
-    itemdata_itemnotes      => $itemfields{itemnotes},
-    itemdata_nonpublicnotes => $itemfields{itemnotes_nonpublic},
     z3950_search_params     => C4::Search::z3950_search_args($dat),
-    hostrecords             => $hostrecords,
-    analytics_flag          => $analytics_flag,
     C4::Search::enabled_staff_search_views,
-    materials => $materials_flag,
 );
 
-if (C4::Context->preference("AlternateHoldingsField") && scalar @items == 0) {
+if (C4::Context->preference("AlternateHoldingsField") && $items_to_display->count == 0) {
     my $fieldspec = C4::Context->preference("AlternateHoldingsField");
     my $subfields = substr $fieldspec, 3;
     my $holdingsep = C4::Context->preference("AlternateHoldingsSeparator") || ' ';
@@ -575,8 +391,6 @@ foreach ( keys %{$dat} ) {
 # method query not found?!?!
 $template->param( AmazonTld => get_amazon_tld() ) if ( C4::Context->preference("AmazonCoverImages"));
 $template->param(
-    itemloop        => \@itemloop,
-    otheritemloop   => \@otheritemloop,
     biblionumber        => $biblionumber,
     ($analyze? 'analyze':'detailview') =>1,
     subscriptions       => \@subs,
@@ -642,7 +456,7 @@ $template->param( holdcount => $holds->count );
 # Check if there are any ILL requests connected to the biblio
 my $illrequests =
     C4::Context->preference('ILLModule')
-  ? Koha::Illrequests->search( { biblio_id => $biblionumber } )
+  ? Koha::ILL::Requests->search( { biblio_id => $biblionumber } )
   : [];
 $template->param( illrequests => $illrequests );
 
@@ -681,6 +495,16 @@ if ( C4::Context->preference('UseCourseReserves') ) {
     $template->param( course_reserves => $course_reserves );
 }
 
+my @libraries = $patron->libraries_where_can_edit_items;
+$template->param(can_edit_items_from => \@libraries);
+
+my @itemtypes = Koha::ItemTypes->search->as_list;
+my %item_type_image_locations = map {
+    $_->itemtype => $_->image_location('intranet')
+} @itemtypes;
+$template->param(item_type_image_locations => \%item_type_image_locations);
+
+
 $template->param(found1 => scalar $query->param('found1') );
 
 $template->param(biblio => $biblio);
