@@ -34,7 +34,7 @@ use Try::Tiny;
 use C4::Output qw( output_and_exit_if_error output_and_exit output_html_with_http_headers );
 use C4::Auth qw( get_session get_template_and_user );
 use C4::Koha;
-use C4::Circulation qw( barcodedecode CanBookBeIssued AddIssue );
+use C4::Circulation qw( barcodedecode CanBookBeIssued AddIssue AddReturn );
 use C4::Members;
 use C4::Biblio qw( TransformMarcToKoha );
 use C4::Search qw( new_record_from_zebra );
@@ -43,6 +43,7 @@ use Koha::Holds;
 use C4::Context;
 use CGI::Session;
 use Koha::AuthorisedValues;
+use Koha::Checkouts::ReturnClaims;
 use Koha::CsvProfiles;
 use Koha::Patrons;
 use Koha::DateUtils qw( dt_from_string );
@@ -111,6 +112,7 @@ my ( $template, $loggedinuser, $cookie ) = get_template_and_user (
     }
 );
 
+my $op                      = $query->param('op') // '';
 my $override_high_holds     = $query->param('override_high_holds');
 my $override_high_holds_tmp = $query->param('override_high_holds_tmp');
 
@@ -131,13 +133,15 @@ if ( C4::Context->preference("AutoSwitchPatron") && $barcode ) {
         $findborrower = $barcode;
         undef $barcode;
         undef $borrowernumber;
+        undef $patron;
+        $barcodes = [];
         $autoswitched = 1;
     }
 }
 $findborrower ||= $query->param('findborrower') || q{};
 $findborrower =~ s|,| |g;
 
-if ( $query->param('confirm_hold') ) {
+if ( $op eq 'cud-confirm_hold' && $query->param('confirm_hold') ) {
     my $reserve_id          = $query->param('confirm_hold');
     my $hold_branch         = $query->param('hold_branch');
     my $hold_itemnumber     = $query->param('hold_itemnumber');
@@ -219,6 +223,10 @@ if( $onsite_checkout && !$duedatespec_allow ) {
         }
     }
 }
+my $reduced_datedue = $query->param('reduceddue');
+if ( $reduced_datedue ) {
+    $datedue = dt_from_string( $reduced_datedue );
+}
 
 my $inprocess = (@$barcodes == 0) ? '' : $query->param('inprocess');
 if ( @$barcodes == 0 && $charges eq 'yes' ) {
@@ -285,6 +293,7 @@ if ($patron) {
         $template->param(
             'userdebarred'    => $patron->debarred,
             'debarredcomment' => $patron->debarredcomment,
+            'debarredsince'   => $patron->restrictions->search()->single->created,
         );
 
         if ( $patron->debarred ne "9999-12-31" ) {
@@ -299,13 +308,18 @@ if ($patron) {
         $template->param( age_high => $patron->category->upperagelimit );
     }
 
+    unless (
+        Koha::Patron::Categories->search_with_library_limits( { 'me.categorycode' => $patron->categorycode } )->count )
+    {
+        $template->param( limited_category => 1 );
+    }
 }
 
 #
 # STEP 3 : ISSUING
 #
 #
-if (@$barcodes) {
+if (@$barcodes && $op eq 'cud-checkout') {
     my $checkout_infos;
     for my $barcode ( @$barcodes ) {
 
@@ -315,9 +329,9 @@ if (@$barcodes) {
         };
 
         # always check for blockers on issuing
-        my ( $error, $question, $alerts, $messages );
+        my ( $issuingimpossible, $needsconfirmation, $alerts, $messages );
         try {
-            ( $error, $question, $alerts, $messages ) = CanBookBeIssued(
+            ( $issuingimpossible, $needsconfirmation, $alerts, $messages ) = CanBookBeIssued(
                 $patron,
                 $barcode, $datedue,
                 $inprocess,
@@ -331,129 +345,200 @@ if (@$barcodes) {
             die $_ unless blessed $_ && $_->can('rethrow');
 
             if ( $_->isa('Koha::Exceptions::Calendar::NoOpenDays') ) {
-                $error = { NO_OPEN_DAYS => 1 };
+                $issuingimpossible = { NO_OPEN_DAYS => 1 };
             } else {
                 $_->rethrow;
             }
         };
 
-    my $blocker = $invalidduedate ? 1 : 0;
+        my $blocker = $invalidduedate ? 1 : 0;
 
-    $template_params->{alert} = $alerts;
-    $template_params->{messages} = $messages;
+        $template_params->{alert} = $alerts;
+        $template_params->{messages} = $messages;
 
-    my $item = Koha::Items->find({ barcode => $barcode });
+        my $item = Koha::Items->find({ barcode => $barcode });
 
-    my $biblio;
-    if ( $item ) {
-        $biblio = $item->biblio;
-    }
+        my $biblio;
+        if ( $item ) {
+            $biblio = $item->biblio;
+        }
 
-    # Fix for bug 7494: optional checkout-time fallback search for a book
+        if ( $issuingimpossible->{'STATS'} ) {
+            my ( $stats_return, $stats_messages, $stats_iteminformation, $stats_borrower ) =
+                AddReturn( $item->barcode, C4::Context->userenv->{'branch'}, undef, undef, 1 );
 
-    if ( $error->{'UNKNOWN_BARCODE'}
-        && C4::Context->preference("itemBarcodeFallbackSearch")
-        && not $batch
-    )
-    {
-     $template_params->{FALLBACK} = 1;
+            $template->param(
+                STATS     => 1,
+                CHECKEDIN => $stats_return,
+                MESSAGES  => $stats_messages,
+                ITEM      => $stats_iteminformation,
+                BORROWER  => $stats_borrower,
+            );
 
-        my $searcher = Koha::SearchEngine::Search->new({index => $Koha::SearchEngine::BIBLIOS_INDEX});
-        my $query = "kw=" . $barcode;
-        my ( $searcherror, $results, $total_hits ) = $searcher->simple_search_compat($query, 0, 10);
+            #increment items.localuse
+            my $localuse_count = $item->localuse;
+            $localuse_count++;
+            $item->localuse($localuse_count)->store;
+        }
 
-        # if multiple hits, offer options to librarian
-        if ( $total_hits > 0 ) {
-            my @barcodes;
-            foreach my $hit ( @{$results} ) {
-                my $chosen = # Maybe easier to retrieve the itemnumber from $hit?
-                  TransformMarcToKoha({ record => C4::Search::new_record_from_zebra('biblioserver',$hit) });
+        # Fix for bug 7494: optional checkout-time fallback search for a book
 
-                # offer all barcodes individually
-                if ( $chosen->{barcode} ) {
-                    push @barcodes, sort split(/\s*\|\s*/, $chosen->{barcode});
+        if ( $issuingimpossible->{'UNKNOWN_BARCODE'}
+            && C4::Context->preference("itemBarcodeFallbackSearch")
+            && not $batch
+        )
+        {
+        $template_params->{FALLBACK} = 1;
+
+            my $searcher = Koha::SearchEngine::Search->new({index => $Koha::SearchEngine::BIBLIOS_INDEX});
+            my $query = "kw=" . $barcode;
+            my ( $searcherror, $results, $total_hits ) = $searcher->simple_search_compat($query, 0, 10);
+
+            # if multiple hits, offer options to librarian
+            if ( $total_hits > 0 ) {
+                my @barcodes;
+                foreach my $hit ( @{$results} ) {
+                    my $chosen = # Maybe easier to retrieve the itemnumber from $hit?
+                    TransformMarcToKoha({ record => C4::Search::new_record_from_zebra('biblioserver',$hit) });
+
+                    # offer all barcodes individually
+                    if ( $chosen->{barcode} ) {
+                        push @barcodes, sort split(/\s*\|\s*/, $chosen->{barcode});
+                    }
+                }
+                my $items = Koha::Items->search({ barcode => {-in => \@barcodes}});
+                $template_params->{options} = $items;
+            }
+        }
+
+        # Only some errors will block when performing forced onsite checkout,
+        # for other cases all errors will block
+        my @blocking_error_codes =
+            ( $onsite_checkout and C4::Context->preference("OnSiteCheckoutsForce") )
+            ? qw( UNKNOWN_BARCODE NO_OPEN_DAYS )
+            : ( keys %$issuingimpossible );
+
+        if ( $issuingimpossible->{BOOKED_TO_ANOTHER} ) {
+            $template_params->{BOOKED_TO_ANOTHER} = $issuingimpossible->{BOOKED_TO_ANOTHER};
+            $template_params->{IMPOSSIBLE}        = 1;
+            $blocker                              = 1;
+        }
+
+        foreach my $code ( @blocking_error_codes ) {
+            if ($issuingimpossible->{$code}) {
+                $template_params->{$code} = $issuingimpossible->{$code};
+
+                $template_params->{IMPOSSIBLE} = 1;
+                $blocker = 1;
+            }
+        }
+
+        delete $needsconfirmation->{'DEBT'} if ($debt_confirmed);
+
+        if ( $item && C4::Context->preference('ClaimReturnedLostValue') ) {
+            my $autoClaimReturnCheckout = C4::Context->preference('AutoClaimReturnStatusOnCheckout');
+
+            my $claims = Koha::Checkouts::ReturnClaims->search(
+                {
+                    itemnumber => $item->id,
+                    resolution => undef,
+                }
+            );
+            if ( $claims->count ) {
+                if ($autoClaimReturnCheckout) {
+                    my $claim = $claims->next;
+
+                    my $patron_id  = $logged_in_user->borrowernumber;
+                    my $resolution = $autoClaimReturnCheckout;
+
+                    $claim->resolve(
+                        {
+                            resolution  => $resolution,
+                            resolved_by => $patron_id,
+                        }
+                    );
+                    $template_params->{CLAIM_RESOLUTION} = $claim;
                 }
             }
-            my $items = Koha::Items->search({ barcode => {-in => \@barcodes}});
-            $template_params->{options} = $items;
         }
-    }
 
-    # Only some errors will block when performing forced onsite checkout,
-    # for other cases all errors will block
-    my @blocking_error_codes =
-        ( $onsite_checkout and C4::Context->preference("OnSiteCheckoutsForce") )
-        ? qw( UNKNOWN_BARCODE NO_OPEN_DAYS )
-        : ( keys %$error );
+        if( $item and ( !$blocker or $force_allow_issue ) ){
+            my $confirm_required = 0;
+            unless($issueconfirmed){
+                #  Get the item title for more information
+                my $materials = $item->materials;
+                my $descriptions = Koha::AuthorisedValues->get_description_by_koha_field({ frameworkcode => $biblio->frameworkcode, kohafield => 'items.materials', authorised_value => $materials });
+                $materials = $descriptions->{lib} // $materials;
+                $template_params->{ADDITIONAL_MATERIALS} = $materials;
+                $template_params->{itemhomebranch} = $item->homebranch;
 
-    foreach my $code ( @blocking_error_codes ) {
-        if ($error->{$code}) {
-            $template_params->{$code} = $error->{$code};
-            $template_params->{IMPOSSIBLE} = 1;
-            $blocker = 1;
-        }
-    }
-
-    delete $question->{'DEBT'} if ($debt_confirmed);
-
-    if( $item and ( !$blocker or $force_allow_issue ) ){
-        my $confirm_required = 0;
-        unless($issueconfirmed){
-            #  Get the item title for more information
-            my $materials = $item->materials;
-            my $descriptions = Koha::AuthorisedValues->get_description_by_koha_field({ frameworkcode => $biblio->frameworkcode, kohafield => 'items.materials', authorised_value => $materials });
-            $materials = $descriptions->{lib} // $materials;
-            $template_params->{ADDITIONAL_MATERIALS} = $materials;
-            $template_params->{itemhomebranch} = $item->homebranch;
-
-            # pass needsconfirmation to template if issuing is possible and user hasn't yet confirmed.
-            foreach my $needsconfirmation ( keys %$question ) {
-                $template_params->{$needsconfirmation} = $$question{$needsconfirmation};
-                $template_params->{getTitleMessageIteminfo} = $biblio->title;
-                $template_params->{getBarcodeMessageIteminfo} = $item->barcode;
-                $template_params->{NEEDSCONFIRMATION} = 1;
-                $confirm_required = 1;
+                # pass needsconfirmation to template if issuing is possible and user hasn't yet confirmed.
+                foreach my $needsconfirmation_key ( keys %$needsconfirmation ) {
+                    $template_params->{$needsconfirmation_key} = $needsconfirmation->{$needsconfirmation_key};
+                    $template_params->{getTitleMessageIteminfo} = $biblio->title;
+                    $template_params->{getBarcodeMessageIteminfo} = $item->barcode;
+                    $template_params->{NEEDSCONFIRMATION} = 1;
+                    $confirm_required = 1;
+                    if ( $needsconfirmation_key eq 'BOOKED_TO_ANOTHER' ) {
+                        my $reduceddue =
+                            dt_from_string( $$needsconfirmation{$needsconfirmation_key}->start_date )->subtract( days => 1 );
+                        $template_params->{reduceddue} = $reduceddue;
+                    }
+                }
             }
-        }
-        unless($confirm_required) {
-            my $switch_onsite_checkout = exists $messages->{ONSITE_CHECKOUT_WILL_BE_SWITCHED};
-            if ( C4::Context->preference('UseRecalls') && !$recall_id ) {
-                my $recall = Koha::Recalls->find(
+            unless ($confirm_required) {
+                my $switch_onsite_checkout = exists $messages->{ONSITE_CHECKOUT_WILL_BE_SWITCHED};
+                if ( C4::Context->preference('UseRecalls') && !$recall_id ) {
+                    my $recall = Koha::Recalls->find(
+                        {
+                            biblio_id => $item->biblionumber,
+                            item_id   => [ undef,       $item->itemnumber ],
+                            status    => [ 'requested', 'waiting' ],
+                            completed => 0,
+                            patron_id => $patron->borrowernumber,
+                        }
+                    );
+                    $recall_id = ( $recall and $recall->id ) ? $recall->id : undef;
+                }
+
+                # If booked (alerts or confirmation) update datedue to end of booking
+                if ( my $booked = $needsconfirmation->{BOOKED_EARLY} // $alerts->{BOOKED} ) {
+                    $datedue = $booked->end_date;
+                }
+                my $issue = AddIssue(
+                    $patron, $barcode, $datedue,
+                    $cancelreserve,
+                    undef, undef,
                     {
-                        biblio_id => $item->biblionumber,
-                        item_id   => [ undef, $item->itemnumber ],
-                        status    => [ 'requested', 'waiting' ],
-                        completed => 0,
-                        patron_id => $patron->borrowernumber,
+                        onsite_checkout        => $onsite_checkout,        auto_renew    => $session->param('auto_renew'),
+                        switch_onsite_checkout => $switch_onsite_checkout, cancel_recall => $cancel_recall,
+                        recall_id              => $recall_id,
                     }
                 );
-                $recall_id = ( $recall and $recall->id ) ? $recall->id : undef;
+                $template_params->{issue} = $issue;
+                $session->clear('auto_renew');
+                $inprocess = 1;
             }
-            my $issue = AddIssue( $patron->unblessed, $barcode, $datedue, $cancelreserve, undef, undef, { onsite_checkout => $onsite_checkout, auto_renew => $session->param('auto_renew'), switch_onsite_checkout => $switch_onsite_checkout, cancel_recall => $cancel_recall, recall_id => $recall_id, } );
-            $template_params->{issue} = $issue;
-            $session->clear('auto_renew');
-            $inprocess = 1;
         }
-    }
 
-    if ($question->{RESERVE_WAITING} or $question->{RESERVED} or $question->{TRANSFERRED} or $question->{PROCESSING}){
-        $template->param(
-            reserveborrowernumber => $question->{'resborrowernumber'},
-            reserve_id => $question->{reserve_id},
-        );
-    }
+        if ($needsconfirmation->{RESERVE_WAITING} or $needsconfirmation->{RESERVED} or $needsconfirmation->{TRANSFERRED} or $needsconfirmation->{PROCESSING}){
+            $template->param(
+                reserveborrowernumber => $needsconfirmation->{'resborrowernumber'},
+                reserve_id => $needsconfirmation->{reserve_id},
+            );
+        }
 
 
-    # FIXME If the issue is confirmed, we launch another time checkouts->count, now display the issue count after issue
-    $patron = Koha::Patrons->find( $borrowernumber );
-    $template_params->{issuecount} = $patron->checkouts->count;
+        # FIXME If the issue is confirmed, we launch another time checkouts->count, now display the issue count after issue
+        $patron = Koha::Patrons->find( $borrowernumber );
+        $template_params->{issuecount} = $patron->checkouts->count;
 
-    if ( $item ) {
-        $template_params->{item} = $item;
-        $template_params->{biblio} = $biblio;
-        $template_params->{itembiblionumber} = $biblio->biblionumber;
-    }
-    push @$checkout_infos, $template_params;
+        if ( $item ) {
+            $template_params->{item} = $item;
+            $template_params->{biblio} = $biblio;
+            $template_params->{itembiblionumber} = $biblio->biblionumber;
+        }
+        push @$checkout_infos, $template_params;
   }
   unless ( $batch ) {
     $template->param( %{$checkout_infos->[0]} );
@@ -461,7 +546,8 @@ if (@$barcodes) {
   } else {
     my $confirmation_needed = grep { $_->{NEEDSCONFIRMATION} } @$checkout_infos;
     $template->param(
-        checkout_infos => $checkout_infos,
+        checkout_infos      => $checkout_infos,
+        onsite_checkout     => $onsite_checkout,
         confirmation_needed => $confirmation_needed,
     );
   }
@@ -581,6 +667,10 @@ if ( $patron ) {
     );
     $template->param( patron_messages => $patron_messages );
 
+    if ( C4::Context->preference("WaitingNotifyAtCheckout") ) {
+        my $waiting_holds = $patron->holds->search( { found => 'W', branchcode => $branch } );
+        $template->param( waiting_holds => $waiting_holds );
+    }
 }
 
 my $fast_cataloging = 0;
@@ -654,7 +744,11 @@ if ( C4::Context->preference("ExportCircHistory") ) {
     $template->param(csv_profiles => Koha::CsvProfiles->search({ type => 'marc' }));
 }
 
-my $has_modifications = Koha::Patron::Modifications->search( { borrowernumber => $borrowernumber } )->count;
+my ( $has_modifications, $patron_lists_count);
+if ( $patron ) {
+    $has_modifications = Koha::Patron::Modifications->search( { borrowernumber => $borrowernumber } )->count;
+    $patron_lists_count = $patron->get_lists_with_patron->count();
+}
 $template->param(
     debt_confirmed            => $debt_confirmed,
     SpecifyDueDate            => $duedatespec_allow,
@@ -662,15 +756,11 @@ $template->param(
     today_due_date_and_time   => dt_from_string()->set(hour => 23)->set(minute => 59),
     restriction_types         => scalar Koha::Patron::Restriction::Types->search(),
     has_modifications         => $has_modifications,
+    patron_lists_count        => $patron_lists_count,
     override_high_holds       => $override_high_holds,
     nopermission              => scalar $query->param('nopermission'),
     autoswitched              => $autoswitched,
     logged_in_user            => $logged_in_user,
 );
 
-# Generate CSRF token for upload and delete image buttons
-$template->param(
-    csrf_token => Koha::Token->new->generate_csrf({ session_id => $query->cookie('CGISESSID'),}),
-);
-
 output_html_with_http_headers $query, $cookie, $template->output;
