@@ -21,7 +21,7 @@
 use Modern::Perl;
 
 use Getopt::Long qw( GetOptions );
-use Pod::Usage qw( pod2usage );
+use Pod::Usage   qw( pod2usage );
 use Text::CSV_XS;
 use DateTime;
 use DateTime::Duration;
@@ -52,7 +52,7 @@ overdue_notices.pl
  Options:
    --help                          Brief help message.
    --man                           Full documentation.
-   --verbose | -v                  Verbose mode. Can be repeated for increased output
+   --verbose | -v                  Verbose mode. Can be repeated for increased output.
    --nomail | -n                   No email will be sent.
    --max          <days>           Maximum days overdue to deal with.
    --library      <branchcode>     Only deal with overdues from this library.
@@ -69,7 +69,7 @@ overdue_notices.pl
    --date         <yyyy-mm-dd>     Emulate overdues run for this date.
    --email        <email_type>     Type of email that will be used.
                                    Can be 'email', 'emailpro' or 'B_email'. Repeatable.
-   --frombranch                    Organize and send overdue notices by home library (item-homebranch) or checkout library (item-issuebranch).
+   --frombranch                    Organize and send overdue notices by home library (item-homebranch) or checkout library (item-issuebranch) or patron home library (patron-homebranch).
                                    This option is only used, if the OverdueNoticeFrom system preference is set to 'command-line option'.
                                    Defaults to item-issuebranch.
 
@@ -89,7 +89,7 @@ Prints the manual page and exits.
 
 Verbose. Without this flag set, only fatal errors are reported.
 A single 'v' will report info on branches, letter codes, and patrons.
-A second 'v' will report The SQL code used to search for triggered patrons.
+A second 'v' will report the SQL code used to search for triggered patrons.
 
 =item B<-n> | B<--nomail>
 
@@ -107,19 +107,19 @@ any CSV files. Defaults to 90 to match F<longoverdues.pl>.
 
 =item B<--library>
 
-select overdues for one specific library. Use the value in the
+Select overdues for one specific library. Use the value in the
 branches.branchcode table. This option can be repeated in order 
 to select overdues for a group of libraries.
 
 =item B<--csv>
 
-Produces CSV data. if -n (no mail) flag is set, then this CSV data is
+Produces CSV data. If -n (no mail) flag is set, then this CSV data is
 sent to standard out or to a filename if provided. Otherwise, only
 overdues that could not be emailed are sent in CSV format to the admin.
 
 =item B<--html>
 
-Produces html data. If patron does not have an email address or
+Produces HTML data. If patron does not have an email address or
 -n (no mail) flag is set, an HTML file is generated in the specified
 directory. This can be downloaded or further processed by library staff.
 The file will be called notices-YYYY-MM-DD.html and placed in the directory
@@ -135,20 +135,24 @@ specified.
 
 =item B<--itemscontent>
 
-comma separated list of fields that get substituted into templates in
+Comma separated list of fields that get substituted into templates in
 places of the E<lt>E<lt>items.contentE<gt>E<gt> placeholder. This
-defaults to due date,title,barcode,author
+defaults to due date,title,barcode,author,itemnumber
 
 Other possible values come from fields in the biblios, items and
 issues tables.
 
 =item B<--borcat>
 
-Repeatable field, that permits to select only some patron categories.
+Repeatable field that permits selecting only specific patron categories.
+For example, --borcat STUDENT --borcat FACULTY would only process overdues
+for patrons in the STUDENT and FACULTY categories.
 
 =item B<--borcatout>
 
-Repeatable field, that permits to exclude some patron categories.
+Repeatable field that permits excluding specific patron categories.
+For example, --borcatout STAFF would skip processing overdues for
+patrons in the STAFF category.
 
 =item B<-t> | B<--triggered>
 
@@ -157,7 +161,7 @@ an item is overdue by the number of days defined in a notice trigger.
 
 By default, a notice is sent each time the script runs, which is suitable for 
 less frequent run cron script, but requires syncing notice triggers with 
-the  cron schedule to ensure proper behavior.
+the cron schedule to ensure proper behavior.
 Add the --triggered option for daily cron, at the risk of no notice 
 being generated if the cron fails to run on time.
 
@@ -165,8 +169,9 @@ being generated if the cron fails to run on time.
 
 This option makes the script run in test mode.
 
-In test mode, the script won't make any changes on the DB. This is useful
-for debugging configuration.
+In test mode, the script won't make any changes to the database. This is useful
+for debugging configuration and testing notice generation without affecting
+production data.
 
 =item B<--list-all>
 
@@ -176,18 +181,33 @@ Choose --list-all to include all overdue items in the list (limited by B<--max>
 
 =item B<--date>
 
-use it in order to send overdues on a specific date and not Now. Format: YYYY-MM-DD.
+Use it in order to send overdues on a specific date and not Now. Format: YYYY-MM-DD.
+
+This allows running the script for a different date than the current one,
+which can be helpful for testing, or for running overdues that should have
+been sent earlier.
 
 =item B<--email>
 
 Allows to specify which type of email will be used. Can be email, emailpro or B_email. Repeatable.
 
+This determines which email address field from the borrowers table will be used
+for notification. If multiple types are specified, the script will try all of them
+in the order provided until it finds a valid address.
+
+If this parameter is not specified, the script will fall back to using Koha::Patron->notice_email_address,
+which respects the patron's preferences and Koha system settings.
+
 =item B<--frombranch>
 
-Organize overdue notices either by checkout library (item-issuebranch) or item home library (item-homebranch).
-This option is only used, if the OverdueNoticeFrom system preference is set to use 'command-line option'.
+Organize overdue notices either by checkout library (item-issuebranch) or item home library (item-homebranch) or patron home library (patron-homebranch).
+This option is only used if the OverdueNoticeFrom system preference is set to use 'command-line option'.
 Defaults to checkout library (item-issuebranch).
 
+This setting affects which branch is considered the "source" of the notice, which
+in turn affects various aspects of the notice, including the content, branding,
+and from email address.
+
 =back
 
 =head1 DESCRIPTION
@@ -280,6 +300,17 @@ items.
 C<overdue_notices.pl --library MAIN max 14> - prepare notices of
 overdues in the last 2 weeks for the MAIN library.
 
+C<overdue_notices.pl --triggered --date 2023-05-01> - Run the script as if it
+were May 1, 2023, and send only notices for items that are exactly at the trigger
+point (rather than within the trigger range).
+
+C<overdue_notices.pl --borcat STUDENT --email email --email B_email> - Only send
+notices to patrons in the STUDENT category, trying both primary and alternate email
+addresses.
+
+C<overdue_notices.pl --html /tmp/notices> - Generate HTML files for patrons without
+email addresses in the /tmp/notices directory.
+
 =head1 SEE ALSO
 
 The F<misc/cronjobs/advance_notices.pl> program allows you to send
@@ -341,11 +372,14 @@ if ( defined $csvfilename && $csvfilename =~ /^-/ ) {
     warn qq(using "$csvfilename" as filename, that seems odd);
 }
 
-die "--frombranch takes item-homebranch or item-issuebranch only"
+die "--frombranch takes item-homebranch or item-issuebranch or patron-homebranch only"
     unless ( $frombranch eq 'item-issuebranch'
-        || $frombranch eq 'item-homebranch' );
-$frombranch = C4::Context->preference('OverdueNoticeFrom') ne 'cron' ? C4::Context->preference('OverdueNoticeFrom') : $frombranch;
+    || $frombranch eq 'item-homebranch'
+    || $frombranch eq 'patron-homebranch' );
+$frombranch =
+    C4::Context->preference('OverdueNoticeFrom') ne 'cron' ? C4::Context->preference('OverdueNoticeFrom') : $frombranch;
 my $owning_library = ( $frombranch eq 'item-homebranch' ) ? 1 : 0;
+my $patron_homelibrary = ( $frombranch eq 'patron-homebranch' ) ? 1 : 0;
 
 my @overduebranches    = C4::Overdues::GetBranchcodesWithOverdueRules();    # Branches with overdue rules
 my @branches;                                    # Branches passed as parameter with overdue rules
@@ -446,6 +480,8 @@ elsif ( defined $text_filename ) {
   }
 }
 
+my %already_queued;
+my %seen = map { $_ => 1 } @branches;
 foreach my $branchcode (@branches) {
     my $calendar;
     if ( C4::Context->preference('OverdueNoticeCalendar') ) {
@@ -476,10 +512,10 @@ SELECT biblio.*, items.*, issues.*, biblioitems.itemtype, branchname
     AND TO_DAYS($date)-TO_DAYS(issues.date_due) >= 0
 END_SQL
 
-    if($owning_library) {
-      $sql2 .= ' AND items.homebranch = ? ';
+    if ($owning_library) {
+        $sql2 .= ' AND items.homebranch = ? ' unless ($patron_homelibrary);
     } else {
-      $sql2 .= ' AND issues.branchcode = ? ';
+        $sql2 .= ' AND issues.branchcode = ? ' unless ($patron_homelibrary);
     }
     my $sth2 = $dbh->prepare($sql2);
 
@@ -489,8 +525,8 @@ END_SQL
     
     my $rqoverduerules =  $dbh->prepare($query);
     $rqoverduerules->execute($branchcode, @myborcat, @myborcatout);
-    
-    # We get default rules is there is no rule for this branch
+
+    # We get default rules if there is no rule for this branch
     if($rqoverduerules->rows == 0){
         $query = "SELECT * FROM overduerules WHERE delay1 IS NOT NULL AND branchcode = '' ";
         $query .= " AND categorycode IN (".join( ',' , ('?') x @myborcat ).") " if (@myborcat);
@@ -595,6 +631,7 @@ END_SQL
                     next;
                 }
                 $borrowernumber = $data->{'borrowernumber'};
+                next if ( $patron_homelibrary && $already_queued{"$borrowernumber$i"} );
                 my $borr = sprintf( "%s%s%s (%s)",
                     $data->{'surname'} || '',
                     $data->{'firstname'} && $data->{'surname'} ? ', ' : '',
@@ -603,6 +640,13 @@ END_SQL
                 $verbose and warn "borrower $borr has items triggering level $i.\n";
 
                 my $patron = Koha::Patrons->find( $borrowernumber );
+                if ($patron_homelibrary) {
+                    $branchcode           = $patron->branchcode;
+                    $library              = Koha::Libraries->find($branchcode);
+                    $admin_email_address  = $library->from_email_address;
+                    $branch_email_address = C4::Context->preference('AddressForFailedOverdueNotices')
+                        || $library->inbound_email_address;
+                }
                 @emails_to_use = ();
                 my $notice_email = $patron->notice_email_address;
                 unless ($nomail) {
@@ -645,7 +689,7 @@ END_SQL
                     ) unless $test_mode;
                     $verbose and warn "debarring $borr\n";
                 }
-                my @params = ($borrowernumber,$branchcode);
+                my @params = $patron_homelibrary ? ($borrowernumber) : ( $borrowernumber, $branchcode );
 
                 $sth2->execute(@params);
                 my $itemcount = 0;
@@ -690,6 +734,7 @@ END_SQL
                       $exceededPrintNoticesMaxLines = 1;
                       last;
                     }
+                    next if $patron_homelibrary and !grep { $seen{ $item_info->{branchcode} } } @branches;
                     $j++;
 
                     $titles .= C4::Letters::get_item_content( { item => $item_info, item_content_fields => \@item_content_fields, dateonly => 1 } );
@@ -813,6 +858,7 @@ END_SQL
                         }
                     }
                 }
+                $already_queued{"$borrowernumber$i"} = 1;
             }
             $sth->finish;
         }
