@@ -48,7 +48,6 @@ use Koha::Patron::Discharge;
 use Koha::Patrons;
 use Koha::Ratings;
 use Koha::Recalls;
-use Koha::Token;
 
 use constant ATTRIBUTE_SHOW_BARCODE => 'SHOW_BCODE';
 
@@ -57,6 +56,8 @@ use Date::Calc qw( Date_to_Days Today );
 
 my $query = CGI->new;
 
+my $op = $query->param('op') || q{};
+
 # CAS single logout handling
 # Will print header and exit
 if ( C4::Context->preference('casAuthentication') ) {
@@ -87,13 +88,7 @@ $template->param( shibbolethAuthentication => C4::Context->config('useshibboleth
 # get borrower information ....
 my $patron = Koha::Patrons->find( $borrowernumber );
 
-if( $query->param('update_arc') && C4::Context->preference("AllowPatronToControlAutorenewal") ){
-    die "Wrong CSRF token"
-        unless Koha::Token->new->check_csrf({
-            session_id => scalar $query->cookie('CGISESSID'),
-            token  => scalar $query->param('csrf_token'),
-        });
-
+if( $op eq 'cud-update_arc' && C4::Context->preference("AllowPatronToControlAutorenewal") ){
     my $autorenew_checkouts = $query->param('borrower_autorenew_checkouts');
     $patron->autorenew_checkouts( $autorenew_checkouts )->store() if defined $autorenew_checkouts;
 }
@@ -166,6 +161,11 @@ if ( $borr->{'dateexpiry'} && C4::Context->preference('NotifyBorrowerDeparture')
     }
 }
 
+my $saving_display = C4::Context->preference('OPACShowSavings');
+if ( $saving_display =~ /user/ ) {
+    $template->param( savings => $patron->get_savings );
+}
+
 # pass on any renew errors to the template for displaying
 my $renew_error = $query->param('renew_error');
 
@@ -173,13 +173,9 @@ $template->param(
                     amountoutstanding => $amountoutstanding,
                     borrowernumber    => $borrowernumber,
                     patron_flagged    => $borr->{flagged},
-                    OPACMySummaryHTML => (C4::Context->preference("OPACMySummaryHTML")) ? 1 : 0,
                     surname           => $borr->{surname},
                     RENEW_ERROR       => $renew_error,
                     borrower          => $borr,
-                    csrf_token             => Koha::Token->new->generate_csrf({
-                        session_id => scalar $query->cookie('CGISESSID'),
-                    }),
                 );
 
 #get issued items ....
@@ -189,7 +185,13 @@ my $overdues_count = 0;
 my @overdues;
 my @issuedat;
 my $itemtypes = { map { $_->{itemtype} => $_ } @{ Koha::ItemTypes->search_with_localization->unblessed } };
-my $pending_checkouts = $patron->pending_checkouts->search({}, { order_by => [ { -desc => 'date_due' }, { -asc => 'issue_id' } ] });
+my $pending_checkouts = $patron->pending_checkouts->search(
+    {},
+    {
+        order_by => [ { -desc => 'date_due' }, { -asc => 'issue_id' } ],
+        prefetch => 'item'
+    }
+);
 my $are_renewable_items = 0;
 if ( $pending_checkouts->count ) { # Useless test
     while ( my $c = $pending_checkouts->next ) {
@@ -222,7 +224,7 @@ if ( $pending_checkouts->count ) { # Useless test
         $issue->{rentalfines} = $rental_fines->total_outstanding;
 
         # check if item is renewable
-        my ($status,$renewerror,$info) = CanBookBeRenewed( $borrowernumber, $issue->{'itemnumber'} );
+        my ($status, $renewerror, $info) = CanBookBeRenewed( $patron, $c );
         (
             $issue->{'renewcount'},
             $issue->{'renewsallowed'},
@@ -230,9 +232,9 @@ if ( $pending_checkouts->count ) { # Useless test
             $issue->{'unseencount'},
             $issue->{'unseenallowed'},
             $issue->{'unseenleft'}
-        ) = GetRenewCount($borrowernumber, $issue->{'itemnumber'});
+        ) = GetRenewCount($patron, $c->item);
         ( $issue->{'renewalfee'}, $issue->{'renewalitemtype'} ) = GetIssuingCharges( $issue->{'itemnumber'}, $borrowernumber );
-        $issue->{itemtype_object} = Koha::ItemTypes->find( Koha::Items->find( $issue->{itemnumber} )->effective_itemtype );
+        $issue->{itemtype_object} = Koha::ItemTypes->find( $c->item->effective_itemtype );
         if($status && C4::Context->preference("OpacRenewalAllowed")){
             $are_renewable_items = 1;
             $issue->{'status'} = $status;
@@ -250,6 +252,7 @@ if ( $pending_checkouts->count ) { # Useless test
             $issue->{'auto_too_late'}  = 1 if $renewerror eq 'auto_too_late';
             $issue->{'auto_too_much_oweing'}  = 1 if $renewerror eq 'auto_too_much_oweing';
             $issue->{'item_denied_renewal'}  = 1 if $renewerror eq 'item_denied_renewal';
+            $issue->{'item_issued_to_other_patron'} = 1 if $renewerror eq 'item_issued_to_other_patron';
 
             if ( $renewerror eq 'too_soon' ) {
                 $issue->{'too_soon'}         = 1;
@@ -294,16 +297,6 @@ if ( $pending_checkouts->count ) { # Useless test
             $issue->{normalized_upc}  = GetNormalizedUPC( $marcrecord, C4::Context->preference('marcflavour') );
             $issue->{normalized_oclc} = GetNormalizedOCLCNumber( $marcrecord, C4::Context->preference('marcflavour') );
         }
-                # My Summary HTML
-                if (my $my_summary_html = C4::Context->preference('OPACMySummaryHTML')){
-                    $issue->{author} ? $my_summary_html =~ s/{AUTHOR}/$issue->{author}/g : $my_summary_html =~ s/{AUTHOR}//g;
-                    $issue->{title} =~ s/\/+$//; # remove trailing slash
-                    $issue->{title} =~ s/\s+$//; # remove trailing space
-                    $issue->{title} ? $my_summary_html =~ s/{TITLE}/$issue->{title}/g : $my_summary_html =~ s/{TITLE}//g;
-                    $issue->{isbn} ? $my_summary_html =~ s/{ISBN}/$isbn/g : $my_summary_html =~ s/{ISBN}//g;
-                    $issue->{biblionumber} ? $my_summary_html =~ s/{BIBLIONUMBER}/$issue->{biblionumber}/g : $my_summary_html =~ s/{BIBLIONUMBER}//g;
-                    $issue->{MySummaryHTML} = $my_summary_html;
-                }
 
         if ( C4::Context->preference('UseRecalls') ) {
             my $maybe_recalls = Koha::Recalls->search({ biblio_id => $issue->{biblionumber}, item_id => [ undef, $issue->{itemnumber} ], completed => 0 });
@@ -408,15 +401,17 @@ if ( C4::Context->preference("ArticleRequests") ) {
 }
 
 $template->param(
-    patron_messages          => $patron_messages,
-    opacnote                 => $borr->{opacnote},
-    patronupdate             => $patronupdate,
-    OpacRenewalAllowed       => C4::Context->preference("OpacRenewalAllowed"),
-    userview                 => 1,
-    SuspendHoldsOpac         => C4::Context->preference('SuspendHoldsOpac'),
-    AutoResumeSuspendedHolds => C4::Context->preference('AutoResumeSuspendedHolds'),
-    OpacHoldNotes            => C4::Context->preference('OpacHoldNotes'),
-    failed_holds             => scalar $query->param('failed_holds'),
+    patron_messages            => $patron_messages,
+    opacnote                   => $borr->{opacnote},
+    patronupdate               => $patronupdate,
+    OpacRenewalAllowed         => C4::Context->preference("OpacRenewalAllowed"),
+    userview                   => 1,
+    SuspendHoldsOpac           => C4::Context->preference('SuspendHoldsOpac'),
+    AutoResumeSuspendedHolds   => C4::Context->preference('AutoResumeSuspendedHolds'),
+    OpacHoldNotes              => C4::Context->preference('OpacHoldNotes'),
+    failed_holds               => scalar $query->param('failed_holds'),
+    opac_user_holds            => scalar $query->param('opac-user-holds')            || 0,
+    opac_user_article_requests => scalar $query->param('opac-user-article-requests') || 0,
 );
 
 # if not an empty string this indicates to return
