@@ -177,7 +177,7 @@ sub build_sample_biblio {
         MARC::Field->new( $tag, ' ', ' ', $subfield => $author ),
     );
 
-    ( $tag, $subfield ) = $marcflavour eq 'UNIMARC' ? ( 995, 'r' ) : ( 942, 'c' );
+    ( $tag, $subfield ) = $marcflavour eq 'UNIMARC' ? ( '099', 't' ) : ( 942, 'c' );
     $record->append_fields(
         MARC::Field->new( $tag, ' ', ' ', $subfield => $itemtype )
     );
@@ -467,6 +467,7 @@ sub _gen_type {
         decimal          => \&_gen_real,
         double_precision => \&_gen_real,
 
+        time      => \&_gen_time,
         timestamp => \&_gen_datetime,
         datetime  => \&_gen_datetime,
         date      => \&_gen_date,
@@ -536,6 +537,11 @@ sub _gen_datetime {
     return $self->schema->storage->datetime_parser->format_datetime(dt_from_string);
 }
 
+sub _gen_time {
+    my ( $self, $params ) = @_;
+    return $self->schema->storage->datetime_parser->format_time(dt_from_string);
+}
+
 sub _gen_text {
     my ($self, $params) = @_;
     # From perldoc String::Random
@@ -588,7 +594,10 @@ sub _gen_default_values {
             debarred       => undef,
             borrowernotes  => '',
             secret         => undef,
+
             password_expiration_date => undef,
+            anonymized               => 0,
+            protected                => 0,
         },
         Item => {
             notforloan         => 0,
@@ -621,10 +630,18 @@ sub _gen_default_values {
         },
         Branch => {
             pickup_location => 0,
+            opacuserjs      => q{},
+            opacusercss     => q{},
         },
         Reserve => {
-            non_priority  => 0,
-            item_group_id => undef,
+            non_priority         => 0,
+            item_group_id        => undef,
+            deleted_biblionumber => undef,
+        },
+        OldReserve => {
+            non_priority         => 0,
+            item_group_id        => undef,
+            deleted_biblionumber => undef,
         },
         Itemtype => {
             rentalcharge => 0,
@@ -648,6 +665,7 @@ sub _gen_default_values {
         },
         BorrowerAttributeType => {
             mandatory => 0,
+            is_date   => 0,
         },
         Suggestion => {
             suggesteddate => dt_from_string()->ymd,
