@@ -56,7 +56,7 @@ $template->param( CanUpdatePasswordExpiration => 1 ) if $logged_in_user->is_supe
 my $dbh       = C4::Context->dbh;
 
 # Show borrower informations
-if ( $op eq 'show' ) {
+if ( $op eq 'cud-show' || $op eq 'show' ) {
     my @borrowers;
     my @patronidnumbers;
     my @notfoundcardnumbers;
@@ -66,6 +66,8 @@ if ( $op eq 'show' ) {
     if( my $cardnumberlist = $input->param('cardnumberlist') ){
         # User submitted a list of card numbers
         push @patronidnumbers, split( /\s\n/, $cardnumberlist );
+    } elsif ( my @cardnumbers = $input->multi_param('cardnumber') ) {
+        @patronidnumbers = @cardnumbers;
     } elsif ( my $cardnumberuploadfile = $input->param('cardnumberuploadfile') ){
         # User uploaded a file of card numbers
         binmode $cardnumberuploadfile, ':encoding(UTF-8)';
@@ -136,9 +138,6 @@ if ( $op eq 'show' ) {
     my $patron_attribute_types = Koha::Patron::Attribute::Types->search_with_library_limits({}, {}, $library_id);
     my @patron_categories = Koha::Patron::Categories->search_with_library_limits({}, {order_by => ['description']})->as_list;
     while ( my $attr_type = $patron_attribute_types->next ) {
-        # TODO Repeatable attributes are not correctly managed and can cause data lost.
-        # This should be implemented.
-        next if $attr_type->repeatable;
         next if $attr_type->unique_id; # Don't display patron attributes that must be unqiue
         my $options = $attr_type->authorised_value_category
             ? GetAuthorisedValues( $attr_type->authorised_value_category )
@@ -150,9 +149,9 @@ if ( $op eq 'show' ) {
             };
 
         my $category_code = $attr_type->category_code;
-        my ( $category_lib ) = map {
-            ( defined $category_code and $attr_type->category_code eq $category_code ) ? $attr_type->description : ()
-        } @patron_categories;
+        my ($category_lib) =
+            map { ( defined $category_code and $attr_type->category_code eq $_->categorycode ) ? $_->description : () }
+            @patron_categories;
         push @patron_attributes_codes,
             {
                 attribute_code => $attr_type->code,
@@ -281,6 +280,12 @@ if ( $op eq 'show' ) {
             mandatory => ( grep /mobile/, @mandatoryFields ) ? 1 : 0,
         }
         ,
+        {
+            name      => "fax",
+            type      => "text",
+            mandatory => ( grep /fax/, @mandatoryFields ) ? 1 : 0,
+        }
+        ,
         {
             name => "sort1",
             type => @sort1_option ? "select" : "text",
@@ -338,17 +343,21 @@ if ( $op eq 'show' ) {
     $template->param('patron_attributes_values', \@patron_attributes_values);
 
     $template->param( fields => \@fields );
+    $op = 'show';
 }
 
 # Process modifications
-if ( $op eq 'do' ) {
+if ( $op eq 'cud-do' ) {
 
     my @disabled = $input->multi_param('disable_input');
     my $infos;
-    for my $field ( qw/surname firstname branchcode categorycode streetnumber address address2 city state zipcode country email phone mobile sort1 sort2 dateenrolled dateexpiry password_expiration_date borrowernotes opacnote debarred debarredcomment/ ) {
+    for my $field (
+        qw/surname firstname branchcode categorycode streetnumber address address2 city state zipcode country email phone mobile fax sort1 sort2 dateenrolled dateexpiry password_expiration_date borrowernotes opacnote debarred debarredcomment/
+        )
+    {
         my $value = $input->param($field);
         $infos->{$field} = $value if $value;
-        $infos->{$field} = "" if grep { $_ eq $field } @disabled;
+        $infos->{$field} = ""     if grep { $_ eq $field } @disabled;
     }
 
     for my $field ( qw( dateenrolled dateexpiry debarred password_expiration_date ) ) {
@@ -357,9 +366,6 @@ if ( $op eq 'do' ) {
 
     delete $infos->{password_expiration_date} unless $logged_in_user->is_superlibrarian;
 
-    my @attributes = $input->multi_param('patron_attributes');
-    my @attr_values = $input->multi_param('patron_attributes_value');
-
     my @errors;
     my @borrowernumbers = $input->multi_param('borrowernumber');
     # For each borrower selected
@@ -398,33 +404,36 @@ if ( $op eq 'do' ) {
         }
 
         my $patron = Koha::Patrons->find( $borrowernumber );
-        my $i=0;
-        for ( @attributes ) {
-            next unless $_;
-            my $attribute;
-            $attribute->{code} = $_;
-            $attribute->{attribute} = $attr_values[$i];
-            my $attr_type = Koha::Patron::Attribute::Types->find($_);
+        my @attributes = $input->multi_param('patron_attributes');
+        my @attr_values = $input->multi_param('patron_attributes_value');
+        my $attributes;
+        my $i = 0;
+        for my $code (@attributes) {
+            push @{ $attributes->{$code}->{values} }, shift @attr_values;    # Handling repeatables
+            $attributes->{$code}->{disabled} = grep { $_ eq sprintf( "attr%s_value", $i++ ) } @disabled;
+        }
+
+        for my $code ( keys %$attributes ) {
+            my $attr_type = Koha::Patron::Attribute::Types->find($code);
+
+            next unless $attr_type;
+
             # If this borrower is not in the category of this attribute, we don't want to modify this attribute
-            ++$i and next if $attr_type->category_code and $attr_type->category_code ne $patron->categorycode;
-            my $valuename = "attr" . $i . "_value";
-            if ( grep { $_ eq $valuename } @disabled ) {
+            next if $attr_type->category_code and $attr_type->category_code ne $patron->categorycode;
+
+            if ( $attributes->{$code}->{disabled} ) {
                 # The attribute is disabled, we remove it for this borrower !
                 eval {
-                    $patron->get_extended_attribute($attribute->{code})->delete;
+                    $patron->extended_attributes->search({'me.code' => $code})->filter_by_branch_limitations->delete;
                 };
                 push @errors, { error => $@ } if $@;
             } else {
                 eval {
-                    # Note:
-                    # We should not need to filter by branch, but stay on the safe side
-                    # Repeatable are not supported so we can do that - TODO
-                    $patron->extended_attributes->search({'me.code' => $attribute->{code}})->filter_by_branch_limitations->delete;
-                    $patron->add_extended_attribute($attribute);
+                    $patron->extended_attributes->search({'me.code' => $code})->filter_by_branch_limitations->delete;
+                    $patron->add_extended_attribute({ code => $code, attribute => $_ }) for @{$attributes->{$code}->{values}};
                 };
                 push @errors, { error => $@ } if $@;
             }
-            $i++;
         }
     }
     $op = "show_results"; # We have process modifications, the user want to view its
